setwd("~/Dropbox/Agriculture Weather/displays")

library(reshape2)
library(dplyr)
library(classInt)
library(maps)
library(maptools)
data(county.fips)

basedir <- "usa cropyield model revised runs May16"
crop.dirs <- c("Code_barley_variance", "Code_corn_variance", "Code_cotton_variance", "Code_rice_variance", "Code_soybean_variance", "Code_wheat_variance")

crops <- c("Barley", "Corn", "Cotton", "Rice", "Soybean", "Wheat")
predfiles <- c("predictors-Barley-combined.csv", "predictors-Maize-combined.csv", "predictors-Cotton-combined.csv", "predictors-Rice-combined.csv", "predictors-Soybeans-combined.csv", "wheat-combine.csv")

fips.usa <- read.csv("../fips_usa.csv")

for (cc in 1:length(crops)) {
    crop <- crops[cc]
    crop.dir <- crop.dirs[cc]

    print(crop)

    df <- read.csv(file.path("~/Dropbox/Agriculture Weather/cropfiles", predfiles[cc]))
    da <- read.table(paste0("../", basedir, "/", crop.dir, "/Ypredict.txt"), header=F)
    names(da) <- fips.usa$FIPS
    da$year <- min(df$year):max(df$year)
    da2 <- melt(da, 'year', variable.name='fips')
    da2$fips <- as.numeric(as.character(da2$fips))

    df2 <- df %>% left_join(da2)

results <- data.frame(fid=c(), rsqr=c())
sstot <- 0
ssres <- 0
for (ii in 1:nrow(fips.usa)) {
    subdf <- subset(df2, fips == fips.usa$FIPS[ii])
    sstotii <- sum((subdf$logy - mean(subdf$logy, na.rm=T))^2, na.rm=T)
    ssresii <- sum((subdf$logy - subdf$value)^2, na.rm=T)
    results <- rbind(results, data.frame(fid=fips.usa$FID[ii], rsqr=1 - ssresii / sstotii))
    if (is.finite(sstotii) && is.finite(ssresii)) {
        sstot <- sstot + sstotii
        ssres <- ssres + ssresii
    }
}

rsqr <- 1 - ssres / sstot
##mean(results$rsqr[is.finite(results$rsqr)], na.rm=T)
##sd(results$rsqr[is.finite(results$rsqr)], na.rm=T)

quantile(results$rsqr, probs=c(.025, .975), na.rm=T)

colors <- c('#9e0142', '#e6f598', '#abdda4', '#66c2a5', '#3288bd', '#5e4fa2')
breaks <- c(-Inf, 0, .2, .4, .6, .8, 1)

colorBuckets <- as.numeric(cut(results$rsqr, breaks))
colorsmatched <- colorBuckets[match(county.fips$fips, fips.usa$FIPS)]

png(paste0("maps/", crop, "-rsqr.png"), width=1200, height=680)
par(mar=c(0, 0, 0, 0))

map("usa")

for (ii in 1:length(colorsmatched)) {
    map("county", county.fips$polyname[ii], col=colors[colorsmatched[ii]], fill=TRUE, resolution=0, add=T, exact=T, lty=0)
}
map("state", col="white", fill=FALSE, add=TRUE, lty=1, lwd=0.2)

legend("bottomleft", legend=leglabs(breaks), fill=colors, bty="n", cex=1.2)

dev.off()

coeffs <- c('alpha', 'beta1', 'beta2', 'beta3', 'beta4')
coeffnames <- c("Intercept", "Technology", "CDI", "GDD", "EDD")

outres <- data.frame(coeff="r2", coeffname="Rsqr", mean=rsqr, sdevmean=sd(results$rsqr, na.rm=T), meanserr=NA, qlo=quantile(results$rsqr, probs=.025, na.rm=T), qhi=quantile(results$rsqr, probs=.975, na.rm=T))
for (ii in 1:length(coeffs)) {
    coeff <- coeffs[ii]
    print(coeff)
    df <- read.table(paste0("../", basedir, "/", crop.dir, "/coeff_", coeff, ".txt"))
    means <- colMeans(df[,1:3111])
    sdevs <- apply(df[,1:3111], 2, sd)

    outres <- rbind(outres, data.frame(coeff, coeffname=coeffnames[ii], mean=mean(means), sdevmean=sd(means), meanserr=mean(sdevs), qlo=quantile(means, probs=.025, na.rm=T), qhi=quantile(means, probs=.975, na.rm=T)))
}

write.csv(outres, file=paste0("tables/", crop, ".csv"), row.names=F)

}

