setwd("~/Dropbox/Agriculture Weather/displays")

library(xtable)

basedir <- "usa cropyield model revised runs May16/Code_"
crop.dirs <- c("barley", "corn", "cotton", "rice", "soybean", "wheat")

crops <- c("Barley", "Corn", "Cotton", "Rice", "Soybean", "Wheat")
irprefs <- c("ba", "cn", "co", "ri", "sy", "wh")

fips <- read.csv("../fips_usa.csv")

bioclim <- read.csv("../us-bioclims-new.csv")
bioclim$fips <- bioclim$STATE * 100 + bioclim$COUNTY / 10

irrig <- read.csv("../irrigation/summary.csv")
irrig$fips <- irrig$STATE * 100 + irrig$COUNTY / 10

order <- sapply(fips$FIPS, function(fip) which(bioclim$fips == fip)[1])

coeffs <- c('alpha', 'beta1', 'beta2', 'beta3', 'beta4')
coeffnames <- c("Intercept", "Technology", "CDI", "GDD", "EDD")

bionames <- c("Mean Temp.", "Isothermality", "Temp. Seasonality", "Precipitation", "Precip. Seasonality")
bioiis <- c(1, 3, 4, 12, 15)

results <- data.frame(crop=c(), coeff=c(), bio1=c(), bio3=c(), bio4=c(), bio12=c(), bio15=c(), irrig=c())

for (cc in 1:6) {
    ircol <- paste0(irprefs[cc], "irsum")
    rncol <- paste0(irprefs[cc], "rnsum")

    valids <- !is.na(irrig[order, ircol]) & irrig[order, ircol] + irrig[order, rncol] > 0

    for (ii in 1:length(coeffs)) {
        coeff <- coeffs[ii]
        print(coeff)
        df <- read.table(paste0("../", basedir, crop.dirs[cc], "/coeff_", coeff, ".txt"))
        means <- colMeans(df[,1:3111])

        moddf <- data.frame(mean=means, irrig=irrig[order, ircol] / (irrig[order, ircol] + irrig[order, rncol]))
        for (bio in bioiis) {
            covar <- paste0("bio", bio, "_mean")
            moddf[, paste0("bio", bio)] <- bioclim[order, covar]
        }

        mod <- lm(mean ~ bio1 + bio3 + bio4 + bio12 + bio15 + irrig, data=moddf[valids,])
        anv <- anova(mod)
        resrow <- anv$`Sum Sq`[-7] / sum(anv$`Sum Sq`)
        results <- rbind(results, data.frame(crop=crops[cc], coeff=coeffnames[ii], bio1=resrow[1], bio3=resrow[2], bio4=resrow[3], bio12=resrow[4], bio15=resrow[5], irrig=resrow[6]))
        print(tail(results))
    }
}

results$tempvex <- rowSums(results[, 3:5])

print(xtable(results[, -which(names(results) == 'tempvex')]), digits=3, include.rownames=F)

results$tempvex[results$coeff == "EDD"]
