## Estimate difference with econometric model
setwd("~/Dropbox/Agriculture Weather/displays")

library(lfe)
library(dplyr)
library(ggplot2)

crops <- c("Barley", "Cotton", "Maize", "Rice", "Soybeans", "Wheat")
crop.dir <- c("Barley", "Cotton", "Corn", "Rice", "Soybean", "Wheat")
predfiles <- c("predictors-Barley-combined.csv", "predictors-Cotton-combined.csv", "predictors-Maize-combined.csv", "predictors-Rice-combined.csv", "predictors-Soybeans-combined.csv", "predictors-Wheat.Winter-combined.csv")

ircols <- c("bairsum", "coirsum", "cnirsum", "riirsum", "syirsum", "whirsum")
rncols <- c("barnsum", "cornsum", "cnrnsum", "rirnsum", "syrnsum", "whrnsum")

bioclim <- read.csv("~/Dropbox/Agriculture Weather/us-bioclims-new.csv")
bioclim$fips <- bioclim$STATE * 100 + bioclim$COUNTY / 10

irrig <- read.csv("~/Dropbox/Agriculture Weather/irrigation/summary.csv")
irrig$fips <- irrig$STATE * 100 + irrig$COUNTY / 10

fips.usa <- read.csv("../posterior_distributions/fips_usa.csv")

mods <- list()
results <- data.frame()
fecompare <- data.frame()
for (ii in 1:6) {
    print(crops[ii])
    predfile <- predfiles[ii]

    df <- read.csv(file.path("~/Dropbox/Agriculture Weather/cropfiles", predfile))
    df <- df %>% left_join(bioclim) %>% left_join(irrig, by="fips")
    df$irfrac <- df[, ircols[ii]] / (df[, ircols[ii]] + df[, rncols[ii]])

    fipsdf <- fips.usa %>% left_join(bioclim, by=c("FIPS"="fips")) %>% left_join(irrig, by=c("FIPS"="fips"))
    fipsdf$irfrac <- fipsdf[, ircols[ii]] / (fipsdf[, ircols[ii]] + fipsdf[, rncols[ii]])
    
    df$year10 <- (df$year - 1948) / 10
    df$gdds1000 <- df$gdds / 1000
    df$kdds1000 <- df$kdds / 1000

    valid <- is.finite(df$logy)
    df <- subset(df, valid)

    mod <- felm(logy ~ year10 + wreq + gdds1000 + kdds1000 + year10 : bio1_mean + year10 : bio3_mean + year10 : bio4_mean + year10 : bio12_mean + year10 : bio15_mean + year10 : irfrac + wreq : bio1_mean + wreq : bio3_mean + wreq : bio4_mean + wreq : bio12_mean + wreq : bio15_mean + wreq : irfrac + gdds1000 : bio1_mean + gdds1000 : bio3_mean + gdds1000 : bio4_mean + gdds1000 : bio12_mean + gdds1000 : bio15_mean + gdds1000 : irfrac + kdds1000 : bio1_mean + kdds1000 : bio3_mean + kdds1000 : bio4_mean + kdds1000 : bio12_mean + kdds1000 : bio15_mean + kdds1000 : irfrac | fips, data=df)
    mods[[ii]] <- mod

    ## Intercepts
    fedf <- getfe(mod)
    fedf$idx <- as.numeric(as.character(fedf$idx))
    fedf <- fedf[, c('effect', 'obs', 'idx')]
    names(fedf) <- c('FE.ols', 'obs', 'idx')
    fipsdf <- fipsdf %>% left_join(fedf, by=c("FIPS"="idx"))
    values <- read.table(paste0("../posterior_distributions/", crop.dir[ii], "/coeff_alpha.txt"), header=F)
    fipsdf <- fipsdf %>% left_join(data.frame(FIPS=fips.usa$FIPS, FE.bay=as.numeric(colMeans(values))))
    results <- rbind(results, cbind(data.frame(crop=crops[ii], coeff="Intercept", median=median(fipsdf$FE.bay)), t(quantile(fipsdf$FE.ols - fipsdf$FE.bay, probs=c(.025, .25, .5, .75, .975), na.rm=T))))
    fecompare <- rbind(fecompare, data.frame(crop=crops[ii], ols=fipsdf$FE.ols, bay=fipsdf$FE.bay))
    
    ## Technology
    rn <- rownames(mod$coefficients)
    
    fipsdf$year10coeff.ols <- mod$coefficients[rn == 'year10'] + fipsdf$bio1_mean * mod$coefficients[rn == 'year10:bio1_mean'] + fipsdf$bio3_mean * mod$coefficients[rn == 'year10:bio3_mean'] + fipsdf$bio4_mean * mod$coefficients[rn == 'year10:bio4_mean'] + fipsdf$bio12_mean * mod$coefficients[rn == 'year10:bio12_mean'] + fipsdf$bio15_mean * mod$coefficients[rn == 'year10:bio15_mean'] + fipsdf$irfrac * mod$coefficients[rn == 'year10:irfrac']
    values <- read.table(paste0("../posterior_distributions/", crop.dir[ii], "/coeff_beta1.txt"), header=F)
    fipsdf <- fipsdf %>% left_join(data.frame(FIPS=fips.usa$FIPS, year10coeff.bay=as.numeric(colMeans(values)) * 10))
    results <- rbind(results, cbind(data.frame(crop=crops[ii], coeff="Trend / decade", median=median(fipsdf$year10coeff.bay)), t(quantile(fipsdf$year10coeff.ols - fipsdf$year10coeff.bay, probs=c(.025, .25, .5, .75, .975), na.rm=T))))

    ## CDI
    fipsdf$wreqcoeff.ols <- mod$coefficients[rn == 'wreq'] + fipsdf$bio1_mean * mod$coefficients[rn == 'wreq:bio1_mean'] + fipsdf$bio3_mean * mod$coefficients[rn == 'wreq:bio3_mean'] + fipsdf$bio4_mean * mod$coefficients[rn == 'wreq:bio4_mean'] + fipsdf$bio12_mean * mod$coefficients[rn == 'wreq:bio12_mean'] + fipsdf$bio15_mean * mod$coefficients[rn == 'wreq:bio15_mean'] + fipsdf$irfrac * mod$coefficients[rn == 'wreq:irfrac']
    values <- read.table(paste0("../posterior_distributions/", crop.dir[ii], "/coeff_beta2.txt"), header=F)
    fipsdf <- fipsdf %>% left_join(data.frame(FIPS=fips.usa$FIPS, wreqcoeff.bay=as.numeric(colMeans(values))))
    results <- rbind(results, cbind(data.frame(crop=crops[ii], coeff="CDI", median=median(fipsdf$wreqcoeff.bay)), t(quantile(fipsdf$wreqcoeff.ols - fipsdf$wreqcoeff.bay, probs=c(.025, .25, .5, .75, .975), na.rm=T))))

    ## GDD
    fipsdf$gdds1000coeff.ols <- mod$coefficients[rn == 'gdds1000'] + fipsdf$bio1_mean * mod$coefficients[rn == 'gdds1000:bio1_mean'] + fipsdf$bio3_mean * mod$coefficients[rn == 'gdds1000:bio3_mean'] + fipsdf$bio4_mean * mod$coefficients[rn == 'gdds1000:bio4_mean'] + fipsdf$bio12_mean * mod$coefficients[rn == 'gdds1000:bio12_mean'] + fipsdf$bio15_mean * mod$coefficients[rn == 'gdds1000:bio15_mean'] + fipsdf$irfrac * mod$coefficients[rn == 'gdds1000:irfrac']
    values <- read.table(paste0("../posterior_distributions/", crop.dir[ii], "/coeff_beta3.txt"), header=F)
    fipsdf <- fipsdf %>% left_join(data.frame(FIPS=fips.usa$FIPS, gdds1000coeff.bay=as.numeric(colMeans(values)) * 1000))
    results <- rbind(results, cbind(data.frame(crop=crops[ii], coeff="1000 GDDs", median=median(fipsdf$gdds1000coeff.bay)), t(quantile(fipsdf$gdds1000coeff.ols - fipsdf$gdds1000coeff.bay, probs=c(.025, .25, .5, .75, .975), na.rm=T))))

    ## KDD
    fipsdf$kdds1000coeff.ols <- mod$coefficients[rn == 'kdds1000'] + fipsdf$bio1_mean * mod$coefficients[rn == 'kdds1000:bio1_mean'] + fipsdf$bio3_mean * mod$coefficients[rn == 'kdds1000:bio3_mean'] + fipsdf$bio4_mean * mod$coefficients[rn == 'kdds1000:bio4_mean'] + fipsdf$bio12_mean * mod$coefficients[rn == 'kdds1000:bio12_mean'] + fipsdf$bio15_mean * mod$coefficients[rn == 'kdds1000:bio15_mean'] + fipsdf$irfrac * mod$coefficients[rn == 'kdds1000:irfrac']
    values <- read.table(paste0("../posterior_distributions/", crop.dir[ii], "/coeff_beta4.txt"), header=F)
    fipsdf <- fipsdf %>% left_join(data.frame(FIPS=fips.usa$FIPS, kdds1000coeff.bay=as.numeric(colMeans(values)) * 1000))
    results <- rbind(results, cbind(data.frame(crop=crops[ii], coeff="1000 KDDs", median=median(fipsdf$kdds1000coeff.bay)), t(quantile(fipsdf$kdds1000coeff.ols - fipsdf$kdds1000coeff.bay, probs=c(.025, .25, .5, .75, .975), na.rm=T))))
    print(tail(results))
}

library(xtable)
print(xtable(results), include.rownames=F)

library(stargazer)
stargazer(mod)

library(ggplot2)
ggplot(fecompare, aes(bay, ols)) +
    facet_wrap(~ crop, scales="free") + geom_point(alpha=.2) +
    xlab("Bayesian intercept") + ylab("OLS intercept") + theme_bw()
ggsave("fecompare.pdf", width=6, height=5)
