setwd("~/Dropbox/Agriculture Weather/displays")

## Run setup from maps.R first

mains <- c("Yields for Soybeans (bu / acre)")
crop <- "Soybean"
filename <- "Ypredict"

predfile <- "predictors-Soybeans-combined.csv"
df <- read.csv(file.path("~/Dropbox/Agriculture Weather/cropfiles", predfile))
df <- subset(df, year >= 1995)

logys <- c()
for (ii in 1:nrow(fips.usa)) {
    logy <- mean(df$logy[df$fips == fips.usa$FIPS[ii]], na.rm=T)
    logys <- c(logys, logy)
}

breaks <- c(0, 1.5, 2.5, 4.5, 7, 12, 20, 33, 55, Inf)
colorBuckets <- as.numeric(cut(exp(logys), breaks))
colorsmatched <- colorBuckets[match(county.fips$fips, fips.usa$FIPS)]

png(paste0("maps/compare-", crop, ".png"), width=1200, height=600)
par(mar=c(0, 0, .5, 1))

map("usa")
for (ii in 1:length(colorsmatched)) {
    map("county", county.fips$polyname[ii], col=colors[[filename]][colorsmatched[ii]], fill=TRUE, resolution=0, add=T, exact=T) #lty=0,
}
map("state", col="white", fill=FALSE, add=TRUE, lty=1, lwd=0.2)

title(mains[filenames == filename], cex.main=2)

legend("bottomleft", legend=leglabs(breaks * scalings[filenames == filename]), fill=colors[[filename]], bty="n", cex=1.2)

dev.off()

alphas <- colMeans(read.table(paste0("../posterior_distributions/", crop.dir, "/coeff_alpha.txt"), header=F))
beta1s <- colMeans(read.table(paste0("../posterior_distributions/", crop.dir, "/coeff_beta1.txt"), header=F))
beta2s <- colMeans(read.table(paste0("../posterior_distributions/", crop.dir, "/coeff_beta2.txt"), header=F))
beta3s <- colMeans(read.table(paste0("../posterior_distributions/", crop.dir, "/coeff_beta3.txt"), header=F))
beta4s <- colMeans(read.table(paste0("../posterior_distributions/", crop.dir, "/coeff_beta4.txt"), header=F))

coeffs <- data.frame(fips=fips.usa$FIPS, alpha=as.vector(alphas), beta1=as.vector(beta1s),
                     beta2=as.vector(beta2s), beta3=as.vector(beta3s), beta4=as.vector(beta4s))

df <- df %>% left_join(coeffs)

library(dplyr)

df$predy <- df$alpha + df$beta1 * df$year + df$beta2 * df$wreq + df$beta3 * df$gdds + df$beta4 * df$kdds

logys <- c()
for (ii in 1:nrow(fips.usa)) {
    logy <- mean(df$predy[df$fips == fips.usa$FIPS[ii]], na.rm=T)
    logys <- c(logys, logy)
}

colorBuckets <- as.numeric(cut(exp(logys), breaks))
colorsmatched <- colorBuckets[match(county.fips$fips, fips.usa$FIPS)]

png(paste0("maps/predict-", crop, ".png"), width=1200, height=600)
par(mar=c(0, 0, .5, 1))

map("usa")
for (ii in 1:length(colorsmatched)) {
    map("county", county.fips$polyname[ii], col=colors[[filename]][colorsmatched[ii]], fill=TRUE, resolution=0, add=T, exact=T) #lty=0,
}
map("state", col="white", fill=FALSE, add=TRUE, lty=1, lwd=0.2)

title(mains[filenames == filename], cex.main=2)

legend("bottomleft", legend=leglabs(breaks * scalings[filenames == filename]), fill=colors[[filename]], bty="n", cex=1.2)

dev.off()
