setwd("~/Dropbox/Agriculture Weather/displays")

library(classInt)
library(maps)
library(maptools)
data(county.fips)

extraneg <- T

knownareas <- read.csv("~/research/water/awash/data/counties/agriculture/knownareas.csv")
knowncrops <- c('BARLEY', 'CORN', 'COTTON', 'RICE', 'SOYBEANS', 'WHEAT')

basedir <- "usa cropyield model revised runs May16"
crop.dirs <- c("barley", "corn", "cotton", "rice", "soybean", "wheat")
sdchange <- c(240.22293, 64.64869, 40.40608, 63.26530, 63.59303, 81.90941) # KDD / SD

include.legend <- F

fips.usa <- read.csv("../fips_usa.csv")
crops <- c("Barley", "Corn", "Cotton", "Rice", "Soybean", "Wheat")
filename <- "coeff_beta4"
main <- "Extreme degree-day coefficient (per 1 SD)"

## Decide on color limits
alldata <- c()
for (crop in crop.dirs) {
    mxf <- read.delim(file.path("~/Dropbox/Agriculture Weather/", basedir, paste0("Code_", crop, "_variance"), paste0(filename, ".txt")), sep=' ', header=F)
    mxf <- unlist(mxf) * sdchange[crop.dirs == crop]
    print(sd(mxf))
    alldata <- c(alldata, mxf)
}

print(classIntervals(alldata, 9, "quantile"))

breaks <- c(-Inf, -.5, -.4, -.3, -.2, -.1, 0, .1, Inf)

greens <- c('#ffffe5','#f7fcb9','#d9f0a3','#addd8e','#78c679','#41ab5d','#238443','#006837','#004529')
browns <- rev(c('#ffffe5', '#fff7bc', '#fee391', '#fec44f', '#fe9929', '#ec7014', '#cc4c02', '#993404', '#662506'))

colors <- c(browns[1], browns[3], browns[4], browns[5], browns[6], browns[7], greens[2], greens[4])

for (cc in (1:length(crops))) {
    crop <- crops[cc]
    crop.dir <- crop.dirs[cc]
    print(crop)
    values <- read.table(paste0("../", basedir, "/", paste0("Code_", crop.dir, "_variance"), "/", filename, ".txt"), header=F)
    origdim <- dim(values)
    if (ncol(values) == 2000)
        values <- values[, 1:1000]

    values.list <- unlist(values) * sdchange[cc]
    values <- matrix(values.list, origdim[1], origdim[2])

    if (extraneg) {
        extracounties <- knownareas$fips[knownareas[, knowncrops[cc]] == 0]
        ispos <- apply(values, 2, function(x) any(x > 0))
        for (ii in which(fips.usa$FIPS %in% extracounties))
            values[values[, ii] > 0, ii] <- NA
        mutedcounties <- fips.usa$FIPS[fips.usa$FIPS %in% extracounties & ispos]
    }

    averages <- as.numeric(colMeans(values, na.rm=T))
    averages[is.na(averages)] <- 0
    colorBuckets <- as.numeric(cut(averages, breaks))
    colorsmatched <- colorBuckets[match(county.fips$fips, fips.usa$FIPS)]

    quant <- apply(values, 2, function(x) quantile(x, probs=c(.025, 1/6, 5/6, .975), na.rm=T))
    border <- colors[colorBuckets]
    border[sign(quant[2,]) != sign(quant[3,])] <- "#FFFFFF"
    border[sign(quant[1,]) == sign(quant[4,])] <- "#000000"
    bordermatched <- border[match(county.fips$fips, fips.usa$FIPS)]

    if (include.legend) {
        png(paste0("maps/", crop, "_variance-", filename, "-sdchange.png"), width=1200, height=600)
        par(mar=c(0, 0, .5, 1))
    } else {
        png(paste0("maps/", crop, "_variance-", filename, "-sdchange-nolegend.png"), width=1200, height=600)
        par(mar=c(0, 0, 0, 0))
    }

    map("usa")
    for (ii in 1:length(colorsmatched)) {
        map("county", county.fips$polyname[ii], col=colors[colorsmatched[ii]], fill=TRUE, resolution=0, add=T, exact=T, lty=0)
        if (extraneg && filename %in% c('coeff_beta2', 'coeff_beta4') && county.fips$fips[ii] %in% mutedcounties) {
            ## map("county", county.fips$polyname[ii], col="#ffffff40", fill=TRUE, resolution=0, add=T, exact=T)
            ## map("county", county.fips$polyname[ii], col="#ffff00", resolution=0, add=T, exact=T)
            map("county", county.fips$polyname[ii], col="#FFFFFF", resolution=0, add=T, exact=T)
        } else if (filename != "Ypredict" && !is.na(bordermatched))
            map("county", county.fips$polyname[ii], col=bordermatched[ii], resolution=0, add=T, exact=T)
    }
    map("state", col="white", fill=FALSE, add=TRUE, lty=1, lwd=0.2)
    if (include.legend) {
        title(main, cex.main=2)

        legend("bottomleft", legend=leglabs(breaks), fill=colors, bty="n", cex=1.2)
    }

    dev.off()
}
