setwd("~/Dropbox/Agriculture Weather/displays")

library(classInt)
library(maps)
library(maptools)
data(county.fips)

extraneg <- T

crop.dirs <- c("barley", "corn", "cotton", "rice", "soybean", "wheat")
crop.dirs2 <- c("Barley", "Corn", "Cotton", "Rice", "Soybean", "Wheat")

basedir <- "usa cropyield model revised runs May16"
fips.usa <- read.csv("../fips_usa.csv")
crops <- c("Barley", "Corn", "Cotton", "Rice", "Soybean", "Wheat")
filenames <- c("Ypredict", "coeff_alpha", "coeff_beta1",
               "coeff_beta2", "coeff_beta3", "coeff_beta4") # "sigma_y"
scalings <- c(1, 1, 10, 1, 1000, 1000) #, 1)
mains <- c("Yields", "Intercept",
           "Technology coefficient (per decade)",
           "Water deficit coefficient",
           "Growing degree-day coefficient (per 1000 GDDs)",
           "Extreme degree-day coefficient (per 1000 EDDs)",
           "Yield uncertainty (log)")
transformlist <- list("Ypredict"=function(x) rank(exp(x)) / length(x),
                      "coeff_alpha"=function(x) x,
                      "coeff_beta1"=function(x) x, "coeff_beta2"=function(x) x,
                      "coeff_beta3"=function(x) x, "coeff_beta4"=function(x) x,
                      "sigma_y"=function(x) x)

knownareas <- read.csv("~/research/water/awash/data/counties/agriculture/knownareas.csv")
knowncrops <- c('BARLEY', 'CORN', 'COTTON', 'RICE', 'SOYBEANS', 'WHEAT')

do.find.breaks <- F

if (do.find.breaks) {
    ## Decide on color limits
    for (filename in filenames) {
        print(filename)
        alldata <- c()
        for (crop in crop.dirs) {
            mxf <- read.delim(file.path("~/Dropbox/Agriculture Weather/", basedir, crop, paste0(filename, ".txt")), sep=' ', header=F)
            mxf <- unlist(mxf)
            mxf <- transformlist[[filename]](mxf)
            alldata <- c(alldata, mxf)
        }

        print(classIntervals(alldata, 9, "quantile"))
    }
}

greens <- c('#ffffe5','#f7fcb9','#d9f0a3','#addd8e','#78c679','#41ab5d','#238443','#006837','#004529')
browns <- rev(c('#ffffe5', '#fff7bc', '#fee391', '#fec44f', '#fe9929', '#ec7014', '#cc4c02', '#993404', '#662506'))

colors <- list("Ypredict"=greens[-9],
               "coeff_alpha"=c(browns[-1], greens[5]),
               "coeff_beta1"=c(browns[5], greens[-9]),
               "coeff_beta2"=c(browns[1], browns[2], browns[4], browns[6], browns[8], greens[2], greens[4], greens[6], greens[8]),
               "coeff_beta3"=c(browns[2], browns[4], browns[6], browns[8], greens[2], greens[4], greens[6], greens[8], greens[9]),
               "coeff_beta4"=c(browns[1], browns[2], browns[3], browns[4], browns[6], browns[8], greens[2], greens[4], greens[6]),
               "sigma_y"=c(greens[7], greens[5], greens[3], greens[1], browns[7], browns[5], browns[3], browns[2], browns[1]))
breakslist <- list("Ypredict"=c(0, .125, .25, .375, .5, .625, .75, .875, 1),
                   "coeff_alpha"=c(-Inf, 0, 1, 2, 3, 4, 5, 6, 7, Inf),
                   "coeff_beta1"=c(-Inf, 0, .005, .0075, .01, .0125, .015, .0175, .02, Inf),
                   "coeff_beta2"=c(-Inf, -.8, -.6, -.4, -.2, 0, .2, .4, .6, Inf),
                   "coeff_beta3"=c(-Inf, -.0015, -.001, -.0005, 0, .0005, .001, .0015, .002, Inf),
                   "coeff_beta4"=c(-Inf, -.005, -.004, -.003, -.002, -.001, 0, .001, .002, Inf),
                   "sigma_y"=c(0, .2, .4, .6, 1, 10, 30, 50, 70, Inf))

for (constvar in c(T, F)) {
    for (include.legend in c(T, F)) {

for (cc in (1:length(crops))) {
    crop <- crops[cc]
    crop.dir <- crop.dirs[cc]
    if (!dir.exists(paste0("../", basedir, "/code_", crop.dir, ifelse(constvar, "_variance", ""))))
        crop.dir <- crop.dirs2[cc]

    for (filename in filenames) {
        print(c(crop, filename))
        values <- read.table(paste0("../", basedir, "/code_", crop.dir, ifelse(constvar, "_variance", ""), "/", filename, ".txt"), header=F)
        origdim <- dim(values)
        if (ncol(values) == 2000)
            values <- values[, 1:1000]

        values.list <- unlist(values)
        values.list <- transformlist[[filename]](values.list)
        values <- matrix(values.list, origdim[1], origdim[2])

        breaks <- breakslist[[filename]]
        if (extraneg && filename %in% c('coeff_beta2', 'coeff_beta4')) {
            extracounties <- knownareas$fips[knownareas[, knowncrops[cc]] == 0]
            ispos <- apply(values, 2, function(x) any(x > 0))
            for (ii in which(fips.usa$FIPS %in% extracounties))
                values[values[, ii] > 0, ii] <- NA
            mutedcounties <- fips.usa$FIPS[fips.usa$FIPS %in% extracounties & ispos]
            print(length(mutedcounties))
        }
        averages <- as.numeric(colMeans(values, na.rm=T))
        averages[is.na(averages)] <- 0
        colorBuckets <- as.numeric(cut(averages, breaks))
        colorsmatched <- colorBuckets[match(county.fips$fips, fips.usa$FIPS)]

        quant <- apply(values, 2, function(x) quantile(x, probs=c(.025, 1/6, 5/6, .975), na.rm=T))
        quant[is.na(quant)] <- 0
        border <- colors[[filename]][colorBuckets]
        border[sign(quant[2,]) != sign(quant[3,])] <- "#FFFFFF"
        border[sign(quant[1,]) == sign(quant[4,])] <- "#000000"
        bordermatched <- border[match(county.fips$fips, fips.usa$FIPS)]

        if (include.legend)
            png(paste0("maps/", crop, ifelse(constvar, "_variance", ""),  "-", filename, ".png"), width=1200, height=600)
        else
            png(paste0("maps/", crop, ifelse(constvar, "_variance", ""), "-", filename, "-nolegend.png"), width=1200, height=600)
        par(mar=c(0, 0, .5, 1))

        map("usa")
        for (ii in 1:length(colorsmatched)) {
            map("county", county.fips$polyname[ii], col=colors[[filename]][colorsmatched[ii]], fill=TRUE, resolution=0, add=T, exact=T) #lty=0,
            if (extraneg && filename %in% c('coeff_beta2', 'coeff_beta4') && county.fips$fips[ii] %in% mutedcounties) {
                map("county", county.fips$polyname[ii], col="#ffffff40", fill=TRUE, resolution=0, add=T, exact=T)
                map("county", county.fips$polyname[ii], col="#ffff00", resolution=0, add=T, exact=T)
            } else if (filename != "Ypredict" && !is.na(bordermatched))
                map("county", county.fips$polyname[ii], col=bordermatched[ii], resolution=0, add=T, exact=T)
        }
        map("state", col="white", fill=FALSE, add=TRUE, lty=1, lwd=0.2)
        if (include.legend) {
            title(mains[filenames == filename], cex.main=2)

            legend("bottomleft", legend=leglabs(breaks * scalings[filenames == filename]), fill=colors[[filename]], bty="n", cex=1.2)
        }

        dev.off()
    }
}
    }
}
