setwd("~/Dropbox/Agriculture Weather/displays")

library(dplyr)
library(scales)
library(RColorBrewer)
library(maps)

basedir <- "usa cropyield model revised runs May16"
fips.usa <- read.csv("../fips_usa.csv")
crop.dirs <- c("barley", "corn", "cotton", "rice", "soybean", "wheat")
crops <- c("Barley", "Corn", "Cotton", "Rice", "Soybean", "Wheat")

predfiles <- c("predictors-Barley-combined.csv", "predictors-Maize-combined.csv", "predictors-Cotton-combined.csv", "predictors-Rice-combined.csv", "predictors-Soybeans-combined.csv", "wheat-combine.csv")

ircols <- c("bairsum", "cnirsum", "coirsum", "riirsum", "syirsum", "whirsum")
rncols <- c("barnsum", "cnrnsum", "cornsum", "rirnsum", "syrnsum", "whrnsum")

bioclim <- read.csv("~/Dropbox/Agriculture Weather/us-bioclims-new.csv")
bioclim$fips <- bioclim$STATE * 100 + bioclim$COUNTY / 10

irrig <- read.csv("~/Dropbox/Agriculture Weather/irrigation/summary.csv")
irrig$fips <- irrig$STATE * 100 + irrig$COUNTY / 10

greens <- c('#ffffe5','#f7fcb9','#d9f0a3','#addd8e','#78c679','#41ab5d','#238443','#006837', '#004529')

show.year <- 2000

do.rand <- F
do.constvar <- T # only relevant if do.rand, F results in a few very large variances

for (cc in 1:length(crops)) {
    crop <- crops[cc]
    crop.dir <- crop.dirs[cc]
    fullcropdir <- paste0("Code_", crop.dir, ifelse(do.constvar, "_variance", ""))

    df <- read.csv(file.path("~/Dropbox/Agriculture Weather/cropfiles", predfiles[cc]))
    if (cc == 6)
        df$year <- df$year + 2010
    df <- subset(df, year == show.year)
    df <- df %>% left_join(bioclim) %>% left_join(irrig, by="fips")
    df$irfrac <- df[, ircols[cc]] / (df[, ircols[cc]] + df[, rncols[cc]])

    df$year10 <- df$year / 10
    df$gdds1000 <- df$gdds / 1000
    df$kdds1000 <- df$kdds / 1000

    ## Sample 4 entries
    rows <- sample(1:3000, 4)

    mu <- matrix(0, 4, 3100)
    for (kk in 0:4) {
        print(kk)

        if (kk == 0) {
            coeffs <- read.table(paste0("../", basedir, "/", fullcropdir, "/coeff_alpha.txt"), header=F)
        } else {
            coeffs <- read.table(paste0("../", basedir, "/", fullcropdir, "/coeff_beta", kk, ".txt"), header=F)
        }

        beta <- as.matrix(coeffs[rows, ])
        beta <- beta[, match(df$fips, fips.usa$FIPS)]

        if (kk == 0) {
            mu <- beta
        } else if (kk == 1) {
            mu <- mu + beta * (show.year - 1948)
        } else {
            for (ii in 1:dim(beta)[2]) {
                if (kk == 2) {
                    mu[, ii] <- mu[, ii] + beta[, ii] * df$wreq[ii]
                } else if (kk == 3) {
                    mu[, ii] <- mu[, ii] + beta[, ii] * df$gdds[ii]
                } else if (kk == 4) {
                    mu[, ii] <- mu[, ii] + beta[, ii] * df$kdds[ii]
                }
            }
        }
    }

    logy <- matrix(NA, 4, 3100)
    if (!do.rand) {
        for (mm in 1:4)
            logy[mm, ] <- mu[mm, ]
    } else {
        sigma <- read.table(paste0("../", basedir, "/", fullcropdir, "/sigma_y.txt"), header=F)
        if (do.constvar) {
            sigma.hat <- sigma[rows, 1]
            for (mm in 1:4)
                logy[mm, ] <- rnorm(dim(beta)[2], mu[mm, ], sigma.hat[mm])
        } else {
            sigma <- sigma[, match(df$fips, fips.usa$FIPS)]
            sigma.hat <- sigma[rows, ]
            for (mm in 1:4)
                logy[mm, ] <- rnorm(dim(beta)[2], as.numeric(mu[mm, ]), as.numeric(sigma.hat[mm, ]))
        }
    }

    limhi <- mean(max(c(max(df$logy, na.rm=T))), max(mu, na.rm=T)) # max(logy, na.rm=T))) or consider
    logy[logy > limhi] <- limhi # for colors
    breaks <- pretty_breaks(10)(c(0, limhi))
    colors <- colorRampPalette(brewer.pal(9, "Greens"))(length(breaks)+1)

    for (mm in 0:4) {
        if (mm == 0) {
            buckets <- as.numeric(cut(df$logy, breaks))
        } else {
            buckets <- as.numeric(cut(logy[mm, ], breaks))
            buckets[is.na(df$logy)] <- NA
        }
        colorsmatched <- buckets[match(county.fips$fips, df$fips)]

        if (mm == 0) {
            png(paste0("postpredmaps", ifelse(do.rand, "-rdrd", "-rdmu"), "/", crop, "-true.png"), width=400, height=200)
        } else {
            png(paste0("postpredmaps", ifelse(do.rand, "-rdrd", "-rdmu"), "/", crop, "-", mm, ".png"), width=400, height=200)
        }
        par(mar=c(0, 0, 0, 0))

        map("usa", mar=rep(0, 4))
        for (ii in 1:length(colorsmatched))
            map("county", county.fips$polyname[ii], col=colors[colorsmatched[ii]], border=colors[colorsmatched[ii]], fill=TRUE, resolution=0, add=T, exact=T)
        dev.off()
    }
}
