setwd("~/Dropbox/Agriculture Weather/displays")

library(ggplot2)

basedir <- "usa cropyield model revised runs May16"
crop.dirs <- paste0("Code_", c("barley", "corn", "cotton", "rice", "soybean", "wheat"), "_variance")
outdir <- "newruns_variance"

cropnames <- c("barley", "corn", "cotton", "rice", "soybeans", "wheat")
crops <- c("Barley", "Corn", "Cotton", "Rice", "Soybean", "Wheat")
irprefs <- c("ba", "cn", "co", "ri", "sy", "wh")

fips <- read.csv("../fips_usa.csv")

bioclim <- read.csv("../us-bioclims-new.csv")
bioclim$fips <- bioclim$STATE * 100 + bioclim$COUNTY / 10

irrig <- read.csv("../irrigation/summary.csv")
irrig$fips <- irrig$STATE * 100 + irrig$COUNTY / 10

order <- sapply(fips$FIPS, function(fip) which(bioclim$fips == fip)[1])

coeffs <- c('alpha', 'beta1', 'beta2', 'beta3', 'beta4')
coeffnames <- c("Intercept", "Technology", "CDI", "GDD", "EDD")

bionames <- c("Mean Temp.", "Diurnal Range", "Isothermality", "Temp. Seasonality", "Max Temp.", "Min Temp.", "Temp. Range", "Wettest Temp.", "Driest Temp.", "Warmest Temp.", "Coldest Temp.", "Precipitation", "Max Precip.", "Min Precip.", "Precip. Seasonality", "Wettest Precip.", "Driest Precip.", "Warmest Precip.", "Coldest Precip.")

dir.create(paste0("scatters/", outdir))

for (cc in 1:6) {

    ircol <- paste0(irprefs[cc], "irsum")
    rncol <- paste0(irprefs[cc], "rnsum")

    ## Plot selected scatters
    df <- read.table(paste0("../", basedir, "/", crop.dirs[cc], "/coeff_", coeffs[5], ".txt"))
    means <- colMeans(df[,1:3111])

    valids <- irrig[order, ircol] + irrig[order, rncol] > 0

    plotdf <- data.frame(eddcoeff=means[valids], covarval=bioclim$bio1_mean[order][valids] / 10, covar="Annual mean temperature (C)")

    plotdf <- rbind(plotdf, data.frame(eddcoeff=means[valids], covarval=irrig[order, ircol][valids] / (irrig[order, ircol][valids] + irrig[order, rncol][valids]), covar="Irrigation fraction"))

    ggplot(plotdf, aes(covarval, eddcoeff * 1000)) +
        facet_grid(~ covar, scales="free") + geom_point(alpha=1/3) +
        ylab("EDD Coefficient (per 1000 degree-days") + xlab("Annual Temperature (C)                                         Irrigation Fraction") +
        theme_bw()
    ggsave(paste0("scatters/", outdir, "/", cropnames[cc], "-tempirr.pdf"), width=7, height=4)

results <- data.frame(coeff=c(), coeffname=c(), covar=c(), covarname=c(), corr=c())
for (ii in 1:length(coeffs)) {
    coeff <- coeffs[ii]
    print(coeff)
    df <- read.table(paste0("../", basedir, "/", crop.dirs[cc], "/coeff_", coeff, ".txt"))
    means <- colMeans(df[,1:3111])

    for (bio in 1:19) {
        covar <- paste0("bio", bio, "_mean")
        results <- rbind(results, data.frame(coeff, coeffname=coeffnames[ii], covar, covarname=bionames[bio], corr=cor(means, bioclim[order, covar], use="complete")))
    }

    results <- rbind(results, data.frame(coeff, coeffname=coeffnames[ii], covar, covarname="Irrigation", corr=cor(means, irrig[order, ircol] / (irrig[order, ircol] + irrig[order, rncol]), use="complete")))
}

results$coeffname <- factor(results$coeffname, rev(coeffnames))

gp <- ggplot(results, aes(covarname, coeffname, fill=corr)) +
    geom_tile(color = "white") +
    scale_fill_gradient2(low = "red", high = "blue", mid = "white",
                      midpoint = 0, limit = c(-1,1), space = "Lab",
                      name="Pearson\nCorrelation") +
    theme_minimal() + geom_text(aes(covarname, coeffname, label=round(results$corr, 1)), color = "black", size = 2) +

    theme(axis.text.x = element_text(angle = 60, vjust = 1,
                                     size = 9, hjust = 1)) +
    coord_fixed() +
    theme(
        axis.title.x = element_blank(),
        axis.title.y = element_blank(),
        panel.grid.major = element_blank(),
        panel.border = element_blank(),
        panel.background = element_blank(),
        axis.ticks = element_blank(),
        legend.position = "bottom",
        legend.direction = "horizontal")+
    guides(fill = guide_colorbar(barwidth = 7, barheight = 1,
                                 title.position = "top", title.hjust = 0.5))
    gp
    ggsave(paste0("correlations/", outdir, "/", cropnames[cc], ".pdf"), width=7, height=4)

    gp + theme(legend.position="none")
    ggsave(paste0("correlations/", outdir, "/", cropnames[cc], "-nolegend.pdf"), width=7, height=2.5)
}
