setwd("~/Dropbox/Agriculture Weather/displays")

library(dplyr)

basedir <- "usa cropyield model revised runs May16"
crop.dirs <- c("Code_barley_variance", "Code_corn_variance", "Code_cotton_variance", "Code_rice_variance", "Code_soybean_variance", "Code_wheat_variance")

fips.usa <- read.csv("../fips_usa.csv")

crops <- c("Barley", "Corn", "Cotton", "Rice", "Soybean", "Wheat")
predfiles <- c("predictors-Barley-combined.csv", "predictors-Maize-combined.csv", "predictors-Cotton-combined.csv", "predictors-Rice-combined.csv", "predictors-Soybeans-combined.csv", "wheat-combine.csv")

get.stats <- function(crop.dir, filename) {
    values <- read.table(paste0("../", basedir, "/", crop.dir, "/", filename, ".txt"), header=F)
    global.mean <- mean(apply(values, 2, mean))
    global.sd <- mean(apply(values, 2, sd))
    quants <- quantile(apply(values, 2, mean), c(.025, .975))
    list(global.mean=global.mean, global.sd=global.sd, quants=quants, byfid=apply(values, 2, mean))
}

format.ms <- function(mu, sigma, digits) {
    paste0(format(round(mu, digits=digits), nsmall=digits, scientific=F), " (", format(round(sigma, digits=digits), nsmall=digits, scientific=F), ")")
}

format.qs <- function(quants, digits) {
    paste0(format(round(quants[1], digits=digits), nsmall=digits, scientific=F), " - ", format(round(quants[2], digits=digits), nsmall=digits, scientific=F))
}

results <- data.frame(crop=c(), rsqr=c(), beta1=c(), beta2=c(), beta3=c(), beta4=c())
eddratio <- data.frame(crop=c(), stderr=c(), cirng=c(), ratio=c())

for (cc in (1:length(crops))) {
    crop <- crops[cc]

    alpha.stats <- get.stats(crop.dirs[cc], "coeff_alpha")
    beta1.stats <- get.stats(crop.dirs[cc], "coeff_beta1")
    beta2.stats <- get.stats(crop.dirs[cc], "coeff_beta2")
    beta3.stats <- get.stats(crop.dirs[cc], "coeff_beta3")
    beta4.stats <- get.stats(crop.dirs[cc], "coeff_beta4")

    df <- read.csv(file.path("~/Dropbox/Agriculture Weather/cropfiles", predfiles[cc]))
    df <- df %>% left_join(fips.usa, by=c("fips"="FIPS"))

    year0 <- min(df$year) - 1
    df$predy <- alpha.stats$byfid[df$FID+1] + beta1.stats$byfid[df$FID+1] * (df$year - year0) + beta2.stats$byfid[df$FID+1] * df$wreq + beta3.stats$byfid[df$FID+1] * df$gdds + beta4.stats$byfid[df$FID+1] * df$kdds

    df <- subset(df, !is.na(logy) & is.finite(logy) & !is.na(predy))

    rsqr <- 1 - sum((df$logy - df$predy)^2) / sum((df$logy - mean(df$logy))^2)

    all.rsqr <- c()
    for (fid in unique(df$FID)) {
        subdf <- subset(df, FID == fid)
        all.rsqr <- c(all.rsqr, 1 - sum((subdf$logy - subdf$predy)^2) / sum((subdf$logy - mean(subdf$logy))^2))
    }

    rquants <- quantile(all.rsqr[is.finite(all.rsqr)], c(.025, .975))

    results <- rbind(results, data.frame(crop, rsqr=format(rsqr, digits=3, nsmall=3), beta1=format.ms(beta1.stats$global.mean, beta1.stats$global.sd, 3), beta2=format.ms(beta2.stats$global.mean, beta2.stats$global.sd, 3), beta3=format.ms(beta3.stats$global.mean, beta3.stats$global.sd, 4), beta4=format.ms(beta4.stats$global.mean, beta4.stats$global.sd, 4)))
    results <- rbind(results, data.frame(crop="\\hspace{.3em}95\\% Range", rsqr=format.qs(rquants, 3), beta1=format.qs(beta1.stats$quants, 3), beta2=format.qs(beta2.stats$quants, 3), beta3=format.qs(beta3.stats$quants, 4), beta4=format.qs(beta4.stats$quants, 4)))
    print(results)

    eddratio <- rbind(eddratio, data.frame(crop, stderr=beta4.stats$global.sd, cirng=diff(beta4.stats$quants), ratio=diff(beta4.stats$quants) / beta4.stats$global.sd))
}

library(xtable)
names(results) <- c("{\\bf Crop}", "{\\bf $R^2$}", "{\\bf Technology}", "{\\bf CDI}", "{\\bf GDD}", "{\\bf EDD}")
print(xtable(results), sanitize.colnames.function=function(x) x, sanitize.text.function=function(x) x, include.rownames=F)

## 95% range vs. std err.
eddratio
