setwd("~/Dropbox/Agriculture Weather/displays")

library(dplyr)

basedir <- "usa cropyield model revised runs May16"
crop.dirs <- c("Code_barley_variance", "Code_corn_variance", "Code_cotton_variance", "Code_rice_variance", "Code_soybean_variance", "Code_wheat_variance")

fips.usa <- read.csv("../fips_usa.csv")

crops <- c("Barley", "Corn", "Cotton", "Rice", "Soybean", "Wheat")
variables <- c("Intercept", "Technology", "CDI", "1000 GDDs", "1000 EDDs")
covariates <- c("Bio 1", "Bio 3", "Bio 4", "Bio 12", "Bio 15", "Irrigation")

format.ms <- function(values, digits) {
    mu <- mean(values)
    if (mu > 0)
        quants <- quantile(values, c(.1, .05, .01))
    else
        quants <- quantile(values, c(.9, .95, .99))
    stars <- paste0(rep('*', sum(sign(quants) == sign(mu))), collapse='')
    paste0(format(round(mean(values), digits=digits), nsmall=digits, scientific=F), stars)
}

format.qs <- function(quants, digits) {
    paste0(format(round(quants[1], digits=digits), nsmall=digits, scientific=F), " - ", format(round(quants[2], digits=digits), nsmall=digits, scientific=F))
}

rownames <- c()
for (var in 1:5) {
    rownames <- c(rownames, variables[var], "")
    for (bio in 1:6)
        rownames <- c(rownames, paste0(" \\hspace{1em} x ", covariates[bio]), "")
}

results <- data.frame(rownames)

for (cc in (1:length(crops))) {
    crop <- crops[cc]

    allvals <- c()
    for (variable in 1:5) {
        values <- read.table(paste0("../", basedir, "/", crop.dirs[cc], "/coeff_a", variable - 1, ".txt"), header=F)
        if (variable == 2)
            values <- values * 10 # /decade
        if (variable %in% c(4, 5))
            values <- values * 1000 # /1000
        allvals <- c(allvals, format.ms(values[, 1], 3),
                     format.qs(quantile(values[, 1], c(.025, .975)), 3))
        values <- read.table(paste0("../", basedir, "/", crop.dirs[cc], "/coeff_b", variable - 1, ".txt"), header=F)
        if (variable == 2)
            values <- values * 10 # /decade
        if (variable %in% c(4, 5))
            values <- values * 1000 # /1000
        values[, 1] <- values[, 1] * 10 # /C
        values[, 2:3] <- values[, 2:3] * 100 # remove 100
        values[, 4:5] <- values[, 4:5] * 1000 # /m
        for (bio in 1:6) {
            allvals <- c(allvals, format.ms(values[, bio], 3),
                         format.qs(quantile(values[, bio], c(.025, .975)), 3))
        }
    }

    results[, crop] <- allvals
    print(results)
}

library(xtable)
print(xtable(results), sanitize.colnames.function=function(x) x, sanitize.text.function=function(x) x, include.rownames=F)
