setwd("~/Dropbox/Agriculture Weather/displays")

library(dplyr)
library(ggplot2)
library(directlabels)

do.drawlegends <- F

crops <- c("Barley", "Corn", "Cotton", "Rice", "Soybean", "Wheat")
predfiles <- c("predictors-Barley-combined.csv", "predictors-Maize-combined.csv", "predictors-Cotton-combined.csv", "predictors-Rice-combined.csv", "predictors-Soybeans-combined.csv", "wheat-combined.csv")

centroids <- read.csv("~/data/political/us-county-centroids.csv") #"~/pCloudDrive/data/political/us-county-centroids.csv")

allsd <- data.frame(crop=c(), mean=c(), sd=c())
for (cc in 1:length(crops)) {
    predfile <- paste0("../cropfiles/", predfiles[cc])
    df <- read.csv(predfile)
    df <- subset(df, year >= 2000)
    allsd <- rbind(allsd, data.frame(crop=crops[cc], mean=mean(df$kdds, na.rm=T), sd=sd(df$kdds, na.rm=T)))
    if (!do.drawlegends)
        next

    df <- df %>% left_join(centroids)
    results <- data.frame(lat0=c(), kdds=c())
    for (lat0 in seq(25, 48, by=2)) {
        kdds <- mean(df$kdds[df$YCOORD >= lat0 & df$YCOORD < lat0+2], na.rm=T)
        results <- rbind(results, data.frame(lat0, kdds))
    }

    pdf(paste0("legends/", crops[cc], "-kdds.pdf"), width=.25, height=2)
    par(mar=c(0, 0, 1, 0))
    plot.new()
    plot.window(c(0, 1), c(25, 49))
    lbls <- contourLines(c(0, 1), results$lat0+1, t(matrix(results$kdds, nrow(results), 2)), nlevels=7)
    contour(c(0, 1), results$lat0+1, t(matrix(results$kdds, nrow(results), 2)), add=T, nlevels=7)
    rect(.15, 25, .85, 49, col="#FFFFFFC0", border=F)
    for (ii in 1:length(lbls))
        text(.5, lbls[[ii]]$y[1], lbls[[ii]]$level, cex=.5)
    title("EDDs", cex.main=.5)
    dev.off()
}
