setwd("~/Dropbox/Agriculture Weather/displays")

library(dplyr)
library(gstat)
library(sp)

basedir <- "usa cropyield model revised runs May16"
crop.dirs <- paste0("Code_", c("barley", "corn", "cotton", "rice", "soybean", "wheat"), "_variance")

crops <- c("Barley", "Corn", "Cotton", "Rice", "Soybean", "Wheat")
filenames <- c("coeff_alpha", "coeff_beta1", "coeff_beta2",
               "coeff_beta3", "coeff_beta4")
mains <- c("Intercept", "Technology coefficient", "Water deficit coefficient",
           "Growing degree-day coefficient", "Extreme degree-day coefficient")

fips <- read.csv("../fips_usa.csv")
centroids <- read.csv("~/pCloud Drive/data/political/us-county-centroids.csv")

results <- data.frame()
plotdf <- data.frame()
for (cc in 1:length(crops)) {
    crop <- crops[cc]
    crop.dir <- crop.dirs[cc]
    for (jj in 1:length(filenames)) {
        filename <- filenames[jj]
        main <- mains[jj]
        print(c(crop, filename))
        values <- read.table(paste0("../", basedir, "/", crop.dir, "/", filename, ".txt"), header=F)
        coeffs <- colMeans(values)[1:3111]
        fips$coeffs <- coeffs
        df <- fips %>% left_join(centroids, by=c("FIPS"="fips"))
        df <- subset(df, !is.na(XCOORD))
        coordinates(df) = ~ XCOORD + YCOORD
        v = variogram(coeffs ~ 1, df, cutoff=40)
        v.fit = fit.variogram(v, vgm("Sph"))
        ## plot(v, v.fit)

        results <- rbind(results, data.frame(crop, filename, main, psill.nug=v.fit[1, 2], psill.sph=v.fit[2, 2], range.sph=v.fit[2, 3], at20=variogramLine(v.fit, 20, n=2)[2,2], mean=mean(coeffs)))

        v.line <- variogramLine(v.fit, 40)
        plotdf <- rbind(plotdf, data.frame(crop, filename, main, xx=v.line[, 1], yy=v.line[, 2]))
    }
}

library(ggplot2)

ggplot(subset(plotdf, filename != "coeff_alpha"), aes(xx * 111, yy, colour=crop)) +
    facet_wrap(~ main, scales='free') +
    geom_line() + geom_point(data=results[results$filename != "coeff_alpha",], aes(4500, mean^2)) +
    ylab("Coefficient Variogram and mean²") + xlab("Distance (km)") + scale_colour_discrete(name="Crops") + theme_minimal()
ggsave("variograms.pdf", width=7, height=4)
