setwd("~/Dropbox/Agriculture Weather/displays")

library(dplyr)
library(gstat)
library(sp)

centroids <- read.csv("~/data/political/us-county-centroids.csv")


filenames <- c('predictors-Barley-combined.csv', 'predictors-Cotton-combined.csv', 'predictors-Maize-combined.csv', 'predictors-Rice-combined.csv', 'predictors-Soybeans-combined.csv', 'predictors-Wheat.Weighted.csv')
crops <- c('Barley', 'Cotton', 'Maize', 'Rice', 'Soybeans', 'Wheat')

results <- data.frame()
for (filename in filenames) {
    df <- read.csv(file.path("../cropfiles", filename))

    df2 <- df %>% left_join(centroids)
    df2 <- subset(df2, !is.na(XCOORD) & !is.na(logy) & is.finite(logy) & year == max(year))
    coordinates(df2) = ~ XCOORD + YCOORD
    v = variogram(logy ~ 1, df2)

    v.fit = fit.variogram(v, vgm("Exp"))
    png(paste0(crops[filenames == filename], ".png"), width=300, height=300)
    plot(v, v.fit, main=crops[filenames == filename])
    dev.off()

    ## Flip it over: varigram(d) = maxvar - cov(d)
    ## varigram(d) = scale * (1 - exp(-d/srate)) + nugget
    ## cov(d) = scale exp(-d/srate)

    results <- rbind(results, data.frame(crop=crops[filenames == filename], nugget=v.fit[1, 2],
                                         scale=v.fit[2, 2], srate=v.fit[2, 3]))

}

## TODO: turn into correlations or actual values that can be used for
## covariance (at x = y, should be variance, not big `scale` value
