# Crop Yield Model using hierarchical Bayesian Estimation - partial pooling #

# Contact: Naresh Devineni, [ndevineni@ccny.cuny.edu] (Civil Engineering, The City University of New York (City College)) 
# Contact: James Rising,(UC Berkley) 

print("This code has xxx stages")
#-------------------------------------------------------------#
# 1: set working directory 

if (Sys.info()[1]=="Darwin")
{
  setwd("/Users/devineni/work/research/Projects/WATER_RISK_ANALYSIS/America_Water_Stress/yield_models/Code_barley_variance/") # Devineni's Mac
} else
{
  setwd("/crest/faculty/ndevineni/usa_cropyield_model/Code_barley_variance") # Devineni's Cluster
}

print (c(Sys.info()[1],"Stage 1 - working directory set"))
#-------------------------------------------------------------#
# 2: Clear the R Workspace  
rm(list = ls(all = TRUE))   

print ("Stage 2 - workspace cleared")
#-------------------------------------------------------------#
# 3: Load Libraries and set working directory 

# Libraries #

# libraries for fitting distributions
library(locfit)
library(logspline)

# libraries for maps
library(maps)
library(maptools)
library(RColorBrewer)
library(classInt)
library(rworldmap)
library(mapproj)

# libraries for running jags in parallel
library(runjags)
library(rjags)
library(R2jags)
library(MCMCpack)
library(parallel)
library(mcmcplots)
library(superdiag)
library(coda)

print ("Stage 3 - all the required libraries are loaded")

#------------------------------------------------------------#
# 4: Read data 

############# crop data ##########
# barley - unit area deficit, yield, gdd and kdd#
barley_yield = (read.csv("barley_yield_in_bu_per_acre.csv",header=T))
barley_ndi = (read.csv("barley_unit_area_deficit_in_meter.csv",header=T))
barley_gddkdd = (read.csv("barley_gddkdd.csv",header=T))

##### Fips Data ####
usa_fips= read.csv("fips_usa.csv")

##### Bioclims Data ####
usa_bioclims= read.csv("bioclims.csv")

##### Irrigation Fraction Data ####
usa_irrfrac= read.csv("irrigation_fraction.csv")

##### Shapefiles for Maps ####

if (Sys.info()[1]=="Darwin")
{
  setwd("/Users/devineni/work/research/Projects/WATER_RISK_ANALYSIS/America_Water_Stress/yield_models/Code_barley_variance/shapefiles") # Devineni's Mac
} else
{
  setwd("/crest/faculty/ndevineni/usa_cropyield_model/Code_barley_variance/shapefiles") # Devineni's Cluster
}

counties.shp <- readShapePoly("counties.shp")
state.shp <- readShapePoly("states.shp")
rivers.shp <- readShapeLines("rivers.shp")
lakes.shp <- readShapePoly("lakes.shp")

if (Sys.info()[1]=="Darwin")
{
  setwd("/Users/devineni/work/research/Projects/WATER_RISK_ANALYSIS/America_Water_Stress/yield_models/Code_barley_variance") # Devineni's Mac
} else
{
  setwd("/crest/faculty/ndevineni/usa_cropyield_model/Code_barley_variance") # Devineni's Cluster
}

print ("Stage 4 - input files are loaded")

#------------------------------------------------------------#
# 5: Arranging the data for consistency 

# Arranging GDD and KDD data #

ndistricts = nrow(barley_yield) # 3111 districts
nyrs = ncol(barley_yield)-4     # 61 years from 1949 - 2009

barley_gdd = matrix(NA,nrow=nyrs,ncol=ndistricts)
barley_kdd = matrix(NA,nrow=nyrs,ncol=ndistricts)

for (i in 1:ndistricts)
  {
    fip = usa_fips[i,1]

    dummy = which(fip==barley_gddkdd[,1])
    if((length(dummy)>0)&(nrow(barley_gddkdd[dummy,])==61)){
    barley_gdd[,i] = barley_gddkdd[dummy,3]
    barley_kdd[,i] = barley_gddkdd[dummy,4]}
  }

# Arranging the bioclims data #

nbios = 19 # 19 bioclimate attributes can be found here: http://www.worldclim.org/bioclim
bios = matrix(NA,nrow=ndistricts,ncol=nbios)

for (i in 1:nrow(usa_bioclims))
{
  #fip_bio = 1000*usa_bioclims[i,]$ID_1/10 + usa_bioclims[i,]$ID_2/10
  fip_bio = usa_bioclims[i,]$fips
  index = which(fip_bio==usa_fips[,1])
  bios[index,1] = usa_bioclims[i,]$bio1mean
  bios[index,2] = usa_bioclims[i,]$bio2mean
  bios[index,3] = usa_bioclims[i,]$bio3mean
  bios[index,4] = usa_bioclims[i,]$bio4mean
  bios[index,5] = usa_bioclims[i,]$bio5mean
  bios[index,6] = usa_bioclims[i,]$bio6mean
  bios[index,7] = usa_bioclims[i,]$bio7mean
  bios[index,8] = usa_bioclims[i,]$bio8mean
  bios[index,9] = usa_bioclims[i,]$bio9mean
  bios[index,10] = usa_bioclims[i,]$bio10mean
  bios[index,11] = usa_bioclims[i,]$bio11mean
  bios[index,12] = usa_bioclims[i,]$bio12mean
  bios[index,13] = usa_bioclims[i,]$bio13mean
  bios[index,14] = usa_bioclims[i,]$bio14mean
  bios[index,15] = usa_bioclims[i,]$bio15mean
  bios[index,16] = usa_bioclims[i,]$bio16mean
  bios[index,17] = usa_bioclims[i,]$bio17mean
  bios[index,18] = usa_bioclims[i,]$bio18mean
  bios[index,19] = usa_bioclims[i,]$bio19mean  
}

# There are some counties with missing bioclims data due to country boundary issues. We identify them and replace  with it with a random county in the state + some noise
dum = which(is.na(bios[,1])==TRUE)

for (i in 1:length(dum))
{
  dummy = which(usa_fips[,4]==usa_fips[dum[i],4])
  dummy2 = intersect(dummy,dum)
  dummy3 = NA
  for (j in 1:length(dummy2))
    {
      dummy3 = c(dummy3,which(dummy==dummy2[j]))
    }
  dummy3 = na.omit(dummy3) 
  random_county_from_state = sample(dummy[-dummy3],1)
  bios[dum[i],]=bios[random_county_from_state,] + rnorm(nbios,0,0.5)
}

### Irrigation Fraction ###
irrigation_fraction = matrix(NA,nrow=ndistricts,ncol=1)

for (i in 1:ndistricts)
{
  fip = usa_fips[i,1]
  
  dummy4 = which(fip==usa_irrfrac[,1])
  irrigation_fraction[i,1] = usa_irrfrac[dummy4,2]
}


bios = cbind(bios[,1],bios[,3],bios[,4],bios[,12],bios[,15],irrigation_fraction)
nbios = ncol(bios)

print ("Stage 5 - data arrangement complete")

#------------------------------------------------------------#
# 6: Basic map of the number of years of data available for each county 

# Number of years of barley data for each county #
barley_years = matrix(NA,nrow=ndistricts,ncol=1)

for (i in 1:ndistricts)
{
  yield_data = as.numeric(barley_yield[i,4:65])
  dummy = na.omit(yield_data)
  barley_years[i,1] = length(which((dummy>0)))  
}

###############################################
# For barley, plot the map of number of years #

pdf("Figure_map_number_of_years.pdf",width=10,height=8)

plotvar = barley_years
nclr = 5
plotclr <- brewer.pal(nclr,"Blues")
#class <- (classIntervals((plotvar), nclr, style="equal"))
class <- classIntervals(plotvar, nclr, style="fixed", fixedBreaks = c(0, 10, 30, 50, 60, 62))
colcode <- (findColours(class, plotclr))
 
map("usa")
map.axes(font=2)
plot(counties.shp,col=colcode, add=T,lty=0)
plot(state.shp,add=T,lwd=1,lty=6);
title(main="Number of Years of Yield Data for barley")
legend("bottomright", legend=names(attr(colcode, "table")), fill=attr(colcode, "palette"), cex=0.9, bty="n")

### End of Plot ###
dev.off()

print ("Stage 6 - basic map complete")

# #------------------------------------------------------------#
# 7: Bayesian Model

# Create the Model for JAGS #
bayes_yield_model <- function() {

  # Model Level 1 #
  for (i in 1:nsites)
  {
    for (t in 1:nyrs)
    {
      Y[t,i] ~ dnorm(alpha[i] + beta1[i]*Technology[t,1] + beta2[i]*NDI[t,i] + beta3[i]*GDD[t,i] + beta4[i]*KDD[t,i],tau.y)
    }
  }
  
  # Model Level 2 #
  for (i in 1:nsites)
  {
    alpha[i] ~ dnorm(a0 + inprod(b0,bios[i,]),tau.alpha)
    beta1[i] ~ dnorm(a1 + inprod(b1,bios[i,]),tau.beta1)
    beta2[i] ~ dnorm(a2 + inprod(b2,bios[i,]),tau.beta2)
    beta3[i] ~ dnorm(a3 + inprod(b3,bios[i,]),tau.beta3)
    beta4[i] ~ dnorm(a4 + inprod(b4,bios[i,]),tau.beta4)
  }
  
  # Priors #
    tau.y <- pow(sigma.y,-2)
    sigma.y ~ dunif(0,100)
    
    #tau.y ~ dgamma (1, 1) 
    #sigma.y <- 1/sqrt(tau.y)

  tau.alpha <- pow(sigma.alpha,-2)
  sigma.alpha ~ dunif(0,100)
  
  #tau.alpha ~ dgamma (1, 1) 
  #sigma.alpha <- 1/sqrt(tau.alpha)
  
  tau.beta1 <- pow(sigma.beta1,-2)
  sigma.beta1 ~ dunif(0,100)
  
  #tau.beta1 ~ dgamma (1, 1) 
  #sigma.beta1 <- 1/sqrt(tau.beta1)
  
  tau.beta2 <- pow(sigma.beta2,-2)
  sigma.beta2 ~ dunif(0,100)
  
  #tau.beta2 ~ dgamma (1, 1) 
  #sigma.beta2 <- 1/sqrt(tau.beta2)
  
  tau.beta3 <- pow(sigma.beta3,-2)
  sigma.beta3 ~ dunif(0,100)
  
  #tau.beta3 ~ dgamma (1, 1) 
  #sigma.beta3 <- 1/sqrt(tau.beta3)
  
  tau.beta4 <- pow(sigma.beta4,-2)
  sigma.beta4 ~ dunif(0,100)
  
  #tau.beta4 ~ dgamma (1, 1) 
  #sigma.beta4 <- 1/sqrt(tau.beta4)
  
  a0 ~ dnorm(0, 0.01)
  a1 ~ dnorm(0, 0.01)
  a2 ~ dnorm(0, 0.01)
  a3 ~ dnorm(0, 0.01)
  a4 ~ dnorm(0, 0.01)
  
  #a3 ~ dunif(0,10)
  #a4 ~ dunif(-10,0)
  
  for (i in 1:nbios)
  {
    b0[i] ~ dnorm(0, 0.01)
    b1[i] ~ dnorm(0, 0.01)
    b2[i] ~ dnorm(0, 0.01)
    b3[i] ~ dnorm(0, 0.01)
    b4[i] ~ dnorm(0, 0.01)
  }

}

print ("Stage 7 - model loaded")

#------------------------------------------------------------#
# 8: Prepare the data for JAGS #

nsites = ndistricts

Y = matrix(NA,nrow=nyrs,ncol=nsites)
#Technology = as.matrix(1949:2009)               # Time trend
Technology = as.matrix(1:61)               # Time trend
NDI = matrix(NA,nrow=nyrs,ncol=nsites) 
GDD = matrix(NA,nrow=nyrs,ncol=nsites) 
KDD = matrix(NA,nrow=nyrs,ncol=nsites) 

for (i in 1:nsites)
{
  Y[,i] = log(as.numeric(barley_yield[i,4:64])) ## yield data from 1949:2009
  NDI[,i] = (as.numeric(barley_ndi[i,5:65])) ## NDI data from 1949:2009
  GDD[,i] = (as.numeric(barley_gdd[,i])) ## GDD data from 1949:2009
  KDD[,i] = (as.numeric(barley_kdd[,i])) ## KDD data from 1949:2009
}

# Replace 0 yield with NA
Y[which(Y==-Inf)] = NA


# Fill NAs in missing GDD value with random county from the State #

for (i in 1:nyrs)
{
  dummy = which(is.na(GDD[i,])=="TRUE")
  
  for (j in 1:length(dummy))
    {
      dummy2 = which(usa_fips[,4]==usa_fips[dummy[j],4])
      dummy3 = intersect(dummy,dummy2)
      
      dummy4 = NA
      for (k in 1:length(dummy3))
      {
        dummy4 = c(dummy4,which(dummy2==dummy3[k]))
      }
      dummy4 = na.omit(dummy4) 
      
      random_county_from_state = sample(dummy2[-dummy4],1)
      GDD[i,dummy[j]] = jitter(GDD[i,random_county_from_state]) 
      KDD[i,dummy[j]] = jitter(KDD[i,random_county_from_state])
      
    }
}


# correlations #
cor_ndi = matrix(NA,nrow=nsites,ncol=1)
cor_gdd = matrix(NA,nrow=nsites,ncol=1)
cor_kdd = matrix(NA,nrow=nsites,ncol=1)

for (i in 1:nsites)
{
  dummy = length(which(Y[,i]>0))
  if (dummy > 0)
  {
    cor_ndi[i,1] = cor(NDI[,i],Y[,i],use="pairwise.complete.obs")
    cor_gdd[i,1] = cor(GDD[,i],Y[,i],use="pairwise.complete.obs")
    cor_kdd[i,1] = cor(KDD[,i],Y[,i],use="pairwise.complete.obs")     
  }
}

  
# Map of the correlation of predictors with log of the yield #

pdf("Figure_map_correlations.pdf",width=10,height=8)

# Correlation with NDI #
plotvar = cor_ndi
nclr = 6
plotclr <- rev(brewer.pal(nclr,"RdBu"))
#class <- (classIntervals((plotvar), nclr, style="equal"))
class <- classIntervals(plotvar, nclr, style="fixed", fixedBreaks = c(-1, -0.5, -0.25, 0, 0.25, 0.5, 1))
colcode <- (findColours(class, plotclr))

map("usa")
map.axes(font=2)
plot(counties.shp,col=colcode, add=T,lty=0)
plot(state.shp,add=T,lwd=1,lty=6);
title(main="Correlation with NDI")
legend("bottomright", legend=names(attr(colcode, "table")), fill=attr(colcode, "palette"), cex=0.9, bty="n")

# Correlation with GDD #
plotvar = cor_gdd
nclr = 6
plotclr <- rev(brewer.pal(nclr,"RdBu"))
#class <- (classIntervals((plotvar), nclr, style="equal"))
class <- classIntervals(plotvar, nclr, style="fixed", fixedBreaks = c(-1, -0.5, -0.25, 0, 0.25, 0.5, 1))
colcode <- (findColours(class, plotclr))

map("usa")
map.axes(font=2)
plot(counties.shp,col=colcode, add=T,lty=0)
plot(state.shp,add=T,lwd=1,lty=6);
title(main="Correlation with GDD")
legend("bottomright", legend=names(attr(colcode, "table")), fill=attr(colcode, "palette"), cex=0.9, bty="n")

# Correlation with KDD #
plotvar = cor_kdd
nclr = 6
plotclr <- rev(brewer.pal(nclr,"RdBu"))
#class <- (classIntervals((plotvar), nclr, style="equal"))
class <- classIntervals(plotvar, nclr, style="fixed", fixedBreaks = c(-1, -0.5, -0.25, 0, 0.25, 0.5, 1))
colcode <- (findColours(class, plotclr))

map("usa")
map.axes(font=2)
plot(counties.shp,col=colcode, add=T,lty=0)
plot(state.shp,add=T,lwd=1,lty=6);
title(main="Correlation with KDD")
legend("bottomright", legend=names(attr(colcode, "table")), fill=attr(colcode, "palette"), cex=0.9, bty="n")

### End of Plot ###
dev.off()

print ("Stage 8 - data prepared for JAGS")
print ("Stage 8 - simple correlation maps done")

#------------------------------------------------------------#
# 9: Run the JAGS Model #
Y_cv = Y[1:46,]
Technology_cv = as.matrix(Technology[1:46,])
NDI_cv = NDI[1:46,]
GDD_cv = GDD[1:46,]
KDD_cv = KDD[1:46,]
nyrs_cv = 46

# send "_cv" data into jags.data for cross-validation #

jags.data <- list('Technology' = Technology,
                  'NDI' = NDI,
                  'GDD' = GDD,
                  'KDD' = KDD,
                  'Y' = Y,
                  'nsites' = nsites,
                  'nyrs'=nyrs,
                  'bios'=bios,
                  'nbios' = nbios)

jags.params <- c('sigma.y',
                 'alpha','beta1','beta2','beta3','beta4',
                 'sigma.alpha','sigma.beta1','sigma.beta2','sigma.beta3','sigma.beta4',
                 'a0','a1','a2','a3','a4',
                 'b0','b1','b2','b3','b4')

inits1 <- list(b0 = rnorm(nbios,0,0.01), b1 = rnorm(nbios,0,0.01), b2 = rnorm(nbios,0,0.01), b3 = rnorm(nbios,0,0.01), b4 = rnorm(nbios,0,0.01),
               a0 = rnorm(nbios,0,0.01), a1 = rnorm(nbios,0,0.01), a2 = rnorm(nbios,0,0.01), a3 = rnorm(nbios,0,0.01), a4 = rnorm(nbios,0,0.01))

inits2 <- list(b0 = rnorm(nbios,0,0.01), b1 = rnorm(nbios,0,0.01), b2 = rnorm(nbios,0,0.01), b3 = rnorm(nbios,0,0.01), b4 = rnorm(nbios,0,0.01),
               a0 = rnorm(nbios,0,0.01), a1 = rnorm(nbios,0,0.01), a2 = rnorm(nbios,0,0.01), a3 = rnorm(nbios,0,0.01), a4 = rnorm(nbios,0,0.01))

inits3 <- list(b0 = rnorm(nbios,0,0.01), b1 = rnorm(nbios,0,0.01), b2 = rnorm(nbios,0,0.01), b3 = rnorm(nbios,0,0.01), b4 = rnorm(nbios,0,0.01),
               a0 = rnorm(nbios,0,0.01), a1 = rnorm(nbios,0,0.01), a2 = rnorm(nbios,0,0.01), a3 = rnorm(nbios,0,0.01), a4 = rnorm(nbios,0,0.01))

	 
#jags.inits <- list(inits1, inits2, inits3)
jags.inits <- NULL

#=============#
# using jags  #
#=============#

# Fit the model using run.jags -- parallel #
bayes.mod.fit  <- jags.parallel(model.file = bayes_yield_model, 
                                data = jags.data, 
                                parameters.to.save = jags.params,
                                inits = jags.inits,
                                n.iter=250000,
				jags.module = c("glm","dic"),
                                n.chains = 3)

attach.jags(bayes.mod.fit)

# Convergence Statistics # 
all_convergence = bayes.mod.fit$BUGSoutput$summary[,8:9]
save.image("output.RData")

print ("Stage 9 - JAGS Model Runs Complete !")

#------------------------------------------------------------#
# 10: Simulate Y.rep data using the parameters  #
nsim = nrow(alpha)

Y.posterior = array(NA,c(nyrs,nsites,nsim))
Ypredict = matrix(NA,nrow=nyrs,ncol=nsites)

for (i in 1:nsites)
  {
    #Ysd = sigma.y[,i]  
      for (j in 1:nyrs)
        {
          Ymean = alpha[,i] + beta1[,i]*Technology[j,1] + beta2[,i]*NDI[j,i] + beta3[,i]*GDD[j,i] + beta4[,i]*KDD[j,i]
          Y.posterior[j,i,] = rnorm(nsim,Ymean,sigma.y)
      }
    Ypredict[,i] = apply(Y.posterior[,i,],1,mean)
  }

print ("Stage 10 - Yrep simulation complete")

#------------------------------------------------------------#
# 11: Writing output files  #

# Level 1 #
write.table(sigma.y,"sigma_y.txt",row.names=F,col.names=F)
write.table(alpha,"coeff_alpha.txt",row.names=F,col.names=F)
write.table(beta1,"coeff_beta1.txt",row.names=F,col.names=F)
write.table(beta2,"coeff_beta2.txt",row.names=F,col.names=F)
write.table(beta3,"coeff_beta3.txt",row.names=F,col.names=F)
write.table(beta4,"coeff_beta4.txt",row.names=F,col.names=F)

# Level 2 #
write.table(sigma.alpha,"sigma_alpha.txt",row.names=F,col.names=F)
write.table(a0,"coeff_a0.txt",row.names=F,col.names=F)
write.table(b0,"coeff_b0.txt",row.names=F,col.names=F)

write.table(sigma.beta1,"sigma_beta1.txt",row.names=F,col.names=F)
write.table(a1,"coeff_a1.txt",row.names=F,col.names=F)
write.table(b1,"coeff_b1.txt",row.names=F,col.names=F)

write.table(sigma.beta2,"sigma_beta2.txt",row.names=F,col.names=F)
write.table(a2,"coeff_a2.txt",row.names=F,col.names=F)
write.table(b2,"coeff_b2.txt",row.names=F,col.names=F)

write.table(sigma.beta3,"sigma_beta3.txt",row.names=F,col.names=F)
write.table(a3,"coeff_a3.txt",row.names=F,col.names=F)
write.table(b3,"coeff_b3.txt",row.names=F,col.names=F)

write.table(sigma.beta4,"sigma_beta4.txt",row.names=F,col.names=F)
write.table(a4,"coeff_a4.txt",row.names=F,col.names=F)
write.table(b4,"coeff_b4.txt",row.names=F,col.names=F)

write.table(Ypredict,"Ypredict.txt", row.names=F,col.names=F)

print ("Stage 11 - writing outputfiles complete")

#------------------------------------------------------------#
# 12: Prepare output figures  #

# Scatterplots of regression coefficients on bioclims #

pdf("Figure_scatter_coefficients.pdf",width=12,height=10)

# 1. Technology Vs. other variables
plotvar = as.numeric(apply(beta1[,1:nsites],2,mean))
par(mfrow=c(2,3))
for (i in 1:nbios)
  {
    biolabel = paste(toString("BIO"),i,sep="")
    plot(bios[,i],plotvar,xlab=biolabel,ylab="Technology Coefficient",font=2,font.lab=2)
    abline(lm(plotvar~bios[,i]),col="grey")  
  }

# 2. NDI Coefficient Vs. other variables
plotvar = as.numeric(apply(beta2[,1:nsites],2,mean))
par(mfrow=c(2,3))
for (i in 1:nbios)
{
  biolabel = paste(toString("BIO"),i,sep="")
  plot(bios[,i],plotvar,xlab=biolabel,ylab="NDI Coefficient",font=2,font.lab=2)  
  abline(lm(plotvar~bios[,i]),col="grey")  
}

# 3. GDD Coefficient Vs. other variables
plotvar = as.numeric(apply(beta3[,1:nsites],2,mean))
par(mfrow=c(2,3))
for (i in 1:nbios)
{
  biolabel = paste(toString("BIO"),i,sep="")
  plot(bios[,i],plotvar,xlab=biolabel,ylab="GDD Coefficient",font=2,font.lab=2)
  abline(lm(plotvar~bios[,i]),col="grey")    
}

# 4. KDD Coefficient Vs. other variables
plotvar = as.numeric(apply(beta4[,1:nsites],2,mean))
par(mfrow=c(2,3))
for (i in 1:nbios)
{
  biolabel = paste(toString("BIO"),i,sep="")
  plot(bios[,i],plotvar,xlab=biolabel,ylab="EDD Coefficient",font=2,font.lab=2)
  abline(lm(plotvar~bios[,i]),col="grey")    
}

dev.off()

# Map of Regression Coefficients (beta1, beta2, beta3, beta4)

pdf("Figure_map_coefficients.pdf",width=10,height=8)

# Coefficient of Technology #
plotvar = as.numeric(apply(beta1[,1:nsites],2,mean))
nclr = 5
plotclr <- (brewer.pal(nclr,"Blues"))
class <- (classIntervals(plotvar, nclr, style="quantile",dataPrecision=3))
colcode <- (findColours(class, plotclr))

map("usa")
map.axes(font=2)
plot(counties.shp,col=colcode, add=T,lty=0)
plot(state.shp,add=T,lwd=1,lty=6);
title(main="Technology Coefficient")
legend("bottomright", legend=names(attr(colcode, "table")), fill=attr(colcode, "palette"), cex=0.9, bty="n")

# Coefficient of NDI #
plotvar = as.numeric(apply(beta2[,1:nsites],2,mean))
nclr = 5
plotclr <- rev(brewer.pal(nclr,"Reds"))
class <- (classIntervals(plotvar, nclr, style="quantile",dataPrecision=3))
colcode <- (findColours(class, plotclr))

map("usa")
map.axes(font=2)
plot(counties.shp,col=colcode, add=T,lty=0)
plot(state.shp,add=T,lwd=1,lty=6);
title(main="NDI Coefficient")
legend("bottomright", legend=names(attr(colcode, "table")), fill=attr(colcode, "palette"), cex=0.9, bty="n")


# Coefficient of GDD #
plotvar = as.numeric(apply(beta3[,1:nsites],2,mean))
nclr = 5
plotclr <- rev(brewer.pal(nclr,"Reds"))
class <- (classIntervals(plotvar, nclr, style="quantile",dataPrecision=3))
colcode <- (findColours(class, plotclr))

map("usa")
map.axes(font=2)
plot(counties.shp,col=colcode, add=T,lty=0)
plot(state.shp,add=T,lwd=1,lty=6);
title(main="GDD Coefficient")
legend("bottomright", legend=names(attr(colcode, "table")), fill=attr(colcode, "palette"), cex=0.9, bty="n")


# Coefficient of EDD #
plotvar = as.numeric(apply(beta4[,1:nsites],2,mean))
nclr = 5
plotclr <- rev(brewer.pal(nclr,"Reds"))
class <- (classIntervals(plotvar, nclr, style="quantile",dataPrecision=3))
colcode <- (findColours(class, plotclr))

map("usa")
map.axes(font=2)
plot(counties.shp,col=colcode, add=T,lty=0)
plot(state.shp,add=T,lwd=1,lty=6);
title(main="EDD Coefficient")
legend("bottomright", legend=names(attr(colcode, "table")), fill=attr(colcode, "palette"), cex=0.9, bty="n")


### End of Plot ###
dev.off()


# boxplots of level2 coefficients for bioclims #

pdf("Figure_boxplot_level2coefficients.pdf",width=12,height=10)

# 1. alpha Vs. bioclims
boxplot(b0,xlab = "Bioclims",ylab="b0 coefficient",font=2,font.lab=2);abline(h=0)

# 2. beta1 Vs. bioclims
boxplot(b1,xlab = "Bioclims",ylab="b1 coefficient",font=2,font.lab=2);abline(h=0)

# 3. beta2 Vs. bioclims
boxplot(b2,xlab = "Bioclims",ylab="b2 coefficient",font=2,font.lab=2);abline(h=0)

# 4. beta3 Vs. bioclims
boxplot(b3,xlab = "Bioclims",ylab="b3 coefficient",font=2,font.lab=2);abline(h=0)

# 5. beta4 Vs. bioclims
boxplot(b4,xlab = "Bioclims",ylab="b4 coefficient",font=2,font.lab=2);abline(h=0)

dev.off()

#Adjusted R2 #

Adjusted_R2full = matrix(NA,nrow=nsites,ncol=1)
for (i in 1:nsites)
  {
    sse = sum((Ypredict[,i] - Y[,i])^2,na.rm=T)
    sst = sum((Y[,i]-mean(Y[,i],na.rm=T))^2,na.rm=T)
    R2 = 1 - (sse/sst)
    dummy = length(which(Y[,i]>0))
    if (dummy>5){Adjusted_R2full[i,1] = 1 - (1-R2)*((nyrs-1)/(nyrs-5)) } 
  }

Adjusted_R2climate = matrix(NA,nrow=nsites,ncol=1)
Ypredictdetrend = matrix(NA,nrow=nyrs,ncol=nsites)
Ydetrend = matrix(NA,nrow=nyrs,ncol=nsites)
for (i in 1:nsites)
  {
    Ytrend = mean(alpha[,i]) + mean(beta1[,i])* (1:61)

    Ydetrend[,i] = Y[,i] - Ytrend
    Ypredictdetrend[,i] = Ypredict[,i] - Ytrend

    sse = sum((Ypredictdetrend[,i] - Ydetrend[,i])^2,na.rm=T)
    sst = sum((Ydetrend[,i]-mean(Ydetrend[,i],na.rm=T))^2,na.rm=T)
    R2 = 1 - (sse/sst)
    dummy = length(which(Y[,i]>0))
    if (dummy>5){Adjusted_R2climate[i,1] = 1 - (1-R2)*((nyrs-1)/(nyrs-5)) } 
  }

# Map of Adjusted R2 #
pdf("Figure_AdjustedR2.pdf",width=12,height=10)

plotvar = Adjusted_R2full
nclr = 5
plotclr <- (brewer.pal(nclr,"Blues"))
class <- (classIntervals(plotvar, nclr, style="quantile",dataPrecision=3))
colcode <- (findColours(class, plotclr))

map("usa")
map.axes(font=2)
plot(counties.shp,col=colcode, add=T,lty=0)
plot(state.shp,add=T,lwd=1,lty=6);
title(main="Adjusted R-square")
legend("bottomright", legend=names(attr(colcode, "table")), fill=attr(colcode, "palette"), cex=0.9, bty="n")



plotvar = Adjusted_R2climate
nclr = 5
plotclr <- (brewer.pal(nclr,"Blues"))
class <- (classIntervals(plotvar, nclr, style="quantile",dataPrecision=2))
colcode <- (findColours(class, plotclr))

map("usa")
map.axes(font=2)
plot(counties.shp,col=colcode, add=T,lty=0)
plot(state.shp,add=T,lwd=1,lty=6);
title(main="Residual Adjusted R-square")
legend("bottomright", legend=names(attr(colcode, "table")), fill=attr(colcode, "palette"), cex=0.9, bty="n")

dev.off()

# Time series plot of selected counties #
pdf("Figure_timeseries_plots.pdf",width=14,height=8)


dummy = which(usa_fips[,4] == 'North Dakota')

i = sample(dummy,1)
name = c(as.character(usa_fips[i,3]),as.character(usa_fips[i,4]))

xticks <- seq(1949, 2009, by = 1)
yticks <- round(seq(min(Y.posterior[,i,]),max(Y.posterior[,i,]),by=1))

boxplot(t(Y.posterior[,i,]),outline=F,ylab="Log(Yield) in Bu/acre",cex.lab=1.2,axes=F,boxwex = 0.1,main=name,cex.main = 1.5,font.lab=2,font.axis=2)
lines(Y[,i],pch=16,col="red",lwd=2)

axis(1, at=seq(1,nyrs,by=1), xticks,las = 1, cex.axis = 1.5,font.lab=2,font.axis=2); 
axis(2,at=round(seq(min(Y.posterior[,i,]),max(Y.posterior[,i,]),by=1)),yticks, las=0, cex.axis = 2,font.lab=2,font.axis=2); box()

dummy = which(usa_fips[,4] == 'South Dakota')

i = sample(dummy,1)
name = c(as.character(usa_fips[i,3]),as.character(usa_fips[i,4]))

xticks <- seq(1949, 2009, by = 1)
yticks <- round(seq(min(Y.posterior[,i,]),max(Y.posterior[,i,]),by=1))

boxplot(t(Y.posterior[,i,]),outline=F,ylab="Log(Yield) in Bu/acre",cex.lab=1.2,axes=F,boxwex = 0.1,main=name,cex.main = 1.5,font.lab=2,font.axis=2)
lines(Y[,i],pch=16,col="red",lwd=2)

axis(1, at=seq(1,nyrs,by=1), xticks,las = 1, cex.axis = 1.5,font.lab=2,font.axis=2); 
axis(2,at=round(seq(min(Y.posterior[,i,]),max(Y.posterior[,i,]),by=1)),yticks, las=0, cex.axis = 2,font.lab=2,font.axis=2); box()

dummy = which(usa_fips[,4] == 'Nebraska')

i = sample(dummy,1)
name = c(as.character(usa_fips[i,3]),as.character(usa_fips[i,4]))

xticks <- seq(1949, 2009, by = 1)
yticks <- round(seq(min(Y.posterior[,i,]),max(Y.posterior[,i,]),by=1))

boxplot(t(Y.posterior[,i,]),outline=F,ylab="Log(Yield) in Bu/acre",cex.lab=1.2,axes=F,boxwex = 0.1,main=name,cex.main = 1.5,font.lab=2,font.axis=2)
lines(Y[,i],pch=16,col="red",lwd=2)

axis(1, at=seq(1,nyrs,by=1), xticks,las = 1, cex.axis = 1.5,font.lab=2,font.axis=2); 
axis(2,at=round(seq(min(Y.posterior[,i,]),max(Y.posterior[,i,]),by=1)),yticks, las=0, cex.axis = 2,font.lab=2,font.axis=2); box()

dummy = which(usa_fips[,4] == 'Iowa')

i = sample(dummy,1)
name = c(as.character(usa_fips[i,3]),as.character(usa_fips[i,4]))

xticks <- seq(1949, 2009, by = 1)
yticks <- round(seq(min(Y.posterior[,i,]),max(Y.posterior[,i,]),by=1))

boxplot(t(Y.posterior[,i,]),outline=F,ylab="Log(Yield) in Bu/acre",cex.lab=1.2,axes=F,boxwex = 0.1,main=name,cex.main = 1.5,font.lab=2,font.axis=2)
lines(Y[,i],pch=16,col="red",lwd=2)

axis(1, at=seq(1,nyrs,by=1), xticks,las = 1, cex.axis = 1.5,font.lab=2,font.axis=2); 
axis(2,at=round(seq(min(Y.posterior[,i,]),max(Y.posterior[,i,]),by=1)),yticks, las=0, cex.axis = 2,font.lab=2,font.axis=2); box()

dummy = which(usa_fips[,4] == 'Missouri')

i = sample(dummy,1)
name = c(as.character(usa_fips[i,3]),as.character(usa_fips[i,4]))

xticks <- seq(1949, 2009, by = 1)
yticks <- round(seq(min(Y.posterior[,i,]),max(Y.posterior[,i,]),by=1))

boxplot(t(Y.posterior[,i,]),outline=F,ylab="Log(Yield) in Bu/acre",cex.lab=1.2,axes=F,boxwex = 0.1,main=name,cex.main = 1.5,font.lab=2,font.axis=2)
lines(Y[,i],pch=16,col="red",lwd=2)

axis(1, at=seq(1,nyrs,by=1), xticks,las = 1, cex.axis = 1.5,font.lab=2,font.axis=2); 
axis(2,at=round(seq(min(Y.posterior[,i,]),max(Y.posterior[,i,]),by=1)),yticks, las=0, cex.axis = 2,font.lab=2,font.axis=2); box()


dummy = which(usa_fips[,4] == 'Illinois')

i = sample(dummy,1)
name = c(as.character(usa_fips[i,3]),as.character(usa_fips[i,4]))

xticks <- seq(1949, 2009, by = 1)
yticks <- round(seq(min(Y.posterior[,i,]),max(Y.posterior[,i,]),by=1))

boxplot(t(Y.posterior[,i,]),outline=F,ylab="Log(Yield) in Bu/acre",cex.lab=1.2,axes=F,boxwex = 0.1,main=name,cex.main = 1.5,font.lab=2,font.axis=2)
lines(Y[,i],pch=16,col="red",lwd=2)

axis(1, at=seq(1,nyrs,by=1), xticks,las = 1, cex.axis = 1.5,font.lab=2,font.axis=2); 
axis(2,at=round(seq(min(Y.posterior[,i,]),max(Y.posterior[,i,]),by=1)),yticks, las=0, cex.axis = 2,font.lab=2,font.axis=2); box()

dummy = which(usa_fips[,4] == 'Ohio')

i = sample(dummy,1)
name = c(as.character(usa_fips[i,3]),as.character(usa_fips[i,4]))

xticks <- seq(1949, 2009, by = 1)
yticks <- round(seq(min(Y.posterior[,i,]),max(Y.posterior[,i,]),by=1))

boxplot(t(Y.posterior[,i,]),outline=F,ylab="Log(Yield) in Bu/acre",cex.lab=1.2,axes=F,boxwex = 0.1,main=name,cex.main = 1.5,font.lab=2,font.axis=2)
lines(Y[,i],pch=16,col="red",lwd=2)

axis(1, at=seq(1,nyrs,by=1), xticks,las = 1, cex.axis = 1.5,font.lab=2,font.axis=2); 
axis(2,at=round(seq(min(Y.posterior[,i,]),max(Y.posterior[,i,]),by=1)),yticks, las=0, cex.axis = 2,font.lab=2,font.axis=2); box()

dummy = which(usa_fips[,4] == 'North Carolina')

i = sample(dummy,1)
name = c(as.character(usa_fips[i,3]),as.character(usa_fips[i,4]))

xticks <- seq(1949, 2009, by = 1)
yticks <- round(seq(min(Y.posterior[,i,]),max(Y.posterior[,i,]),by=1))

boxplot(t(Y.posterior[,i,]),outline=F,ylab="Log(Yield) in Bu/acre",cex.lab=1.2,axes=F,boxwex = 0.1,main=name,cex.main = 1.5,font.lab=2,font.axis=2)
lines(Y[,i],pch=16,col="red",lwd=2)

axis(1, at=seq(1,nyrs,by=1), xticks,las = 1, cex.axis = 1.5,font.lab=2,font.axis=2); 
axis(2,at=round(seq(min(Y.posterior[,i,]),max(Y.posterior[,i,]),by=1)),yticks, las=0, cex.axis = 2,font.lab=2,font.axis=2); box()


dummy = which(usa_fips[,4] == 'Washington')

i = sample(dummy,1)
name = c(as.character(usa_fips[i,3]),as.character(usa_fips[i,4]))

xticks <- seq(1949, 2009, by = 1)
yticks <- round(seq(min(Y.posterior[,i,]),max(Y.posterior[,i,]),by=1))

boxplot(t(Y.posterior[,i,]),outline=F,ylab="Log(Yield) in Bu/acre",cex.lab=1.2,axes=F,boxwex = 0.1,main=name,cex.main = 1.5,font.lab=2,font.axis=2)
lines(Y[,i],pch=16,col="red",lwd=2)

axis(1, at=seq(1,nyrs,by=1), xticks,las = 1, cex.axis = 1.5,font.lab=2,font.axis=2); 
axis(2,at=round(seq(min(Y.posterior[,i,]),max(Y.posterior[,i,]),by=1)),yticks, las=0, cex.axis = 2,font.lab=2,font.axis=2); box()


dummy = which(usa_fips[,4] == 'New Mexico')

i = sample(dummy,1)
name = c(as.character(usa_fips[i,3]),as.character(usa_fips[i,4]))

xticks <- seq(1949, 2009, by = 1)
yticks <- round(seq(min(Y.posterior[,i,]),max(Y.posterior[,i,]),by=1))

boxplot(t(Y.posterior[,i,]),outline=F,ylab="Log(Yield) in Bu/acre",cex.lab=1.2,axes=F,boxwex = 0.1,main=name,cex.main = 1.5,font.lab=2,font.axis=2)
lines(Y[,i],pch=16,col="red",lwd=2)

axis(1, at=seq(1,nyrs,by=1), xticks,las = 1, cex.axis = 1.5,font.lab=2,font.axis=2); 
axis(2,at=round(seq(min(Y.posterior[,i,]),max(Y.posterior[,i,]),by=1)),yticks, las=0, cex.axis = 2,font.lab=2,font.axis=2); box()


dev.off()

print ("Stage 12 - Plots complete")


#------------------------------------------------------------#

print("-------------This is the end of the Code-----------")




