# Crop Yield Model using hierarchical Bayesian Estimation - partial pooling #

# Posterior Predictive Checks #

# After reading the .RData file # 

# libraries for running jags in parallel
library(runjags)
library(rjags)
library(R2jags)
library(MCMCpack)
library(parallel)
library(mcmcplots)
library(superdiag)
library(coda)

# libraries for maps
library(maps)
library(maptools)
library(RColorBrewer)
library(classInt)
library(rworldmap)
library(mapproj)

attach.jags(bayes.mod.fit)

# Run Y.posterior after these lines # 
# 10: Simulate Y.rep data using the parameters   -- full model #
{
    nsim = nrow(alpha)
    
    Y.posterior = array(NA,c(nyrs,nsites,nsim))
    Ypredict = matrix(NA,nrow=nyrs,ncol=nsites)
    
    for (i in 1:nsites)
    {
      Ysd = sigma.y[,i]  
      for (j in 1:nyrs)
      {
        Ymean = alpha[,i] + beta1[,i]*Technology[j,1] + beta2[,i]*NDI[j,i] + beta3[,i]*GDD[j,i] + beta4[,i]*KDD[j,i]
        Y.posterior[j,i,] = rnorm(nsim,Ymean,Ysd)
      }
      Ypredict[,i] = apply(Y.posterior[,i,],1,mean)
    }
}


# 10: Simulate Y.rep data using the parameters  - for variance model #
{
    nsim = nrow(alpha)
    
    Y.posterior = array(NA,c(nyrs,nsites,nsim))
    Ypredict = matrix(NA,nrow=nyrs,ncol=nsites)
    
    for (i in 1:nsites)
    {
      #Ysd = sigma.y[,i]  
      for (j in 1:nyrs)
      {
        Ymean = alpha[,i] + beta1[,i]*Technology[j,1] + beta2[,i]*NDI[j,i] + beta3[,i]*GDD[j,i] + beta4[,i]*KDD[j,i]
        Y.posterior[j,i,] = rnorm(nsim,Ymean,sigma.y)
      }
      Ypredict[,i] = apply(Y.posterior[,i,],1,mean)
    }
}

#------------------------------------------------------------#
# Bayes P-value for T(y) = y[10] and y[90]-y[10] Values # 
ind = 47:nyrs

y10 = apply(Y[ind,],2,quantile,0.1,na.rm=T)
yrange = apply(Y[ind,],2,quantile,0.9,na.rm=T) - apply(Y[ind,],2,quantile,0.1,na.rm=T)

ind_counties = which(is.na(y10)==FALSE)
y10 = y10[ind_counties]
yrange = yrange[ind_counties]

N = 500
random_sims = sample(1:nsim,N,replace=FALSE)

y10_rep = matrix(NA,nrow=N,ncol=length(ind_counties))
yrange_rep = matrix(NA,nrow=N,ncol=length(ind_counties))

for(i in 1:N)
  {
    y10_rep[i,] = apply(Y.posterior[ind,ind_counties,random_sims[i]],2,quantile,0.1)
    yrange_rep[i,] = apply(Y.posterior[ind,ind_counties,random_sims[i]],2,quantile,0.9) - apply(Y.posterior[ind,ind_counties,random_sims[i]],2,quantile,0.1)
  }

bayespval_y10 = matrix(NA,nrow=1,ncol=length(ind_counties))
bayespval_yrange = matrix(NA,nrow=1,ncol=length(ind_counties))

for (i in 1:length(ind_counties))
{
    bayespval_y10[1,i] = length(which(y10_rep[,i] >= y10[i]))/N
    bayespval_yrange[1,i] = length(which(yrange_rep[,i] >= yrange[i]))/N
}
bayespval_y10_summary = c(min(bayespval_y10,na.rm=T),quantile(bayespval_y10,c(0.01,0.1,0.25,0.5,0.75,0.9,0.99),na.rm=T),max(bayespval_y10,na.rm=T),mean(bayespval_y10,na.rm=T))
round(bayespval_y10_summary,2)

bayespval_yrange_summary = c(min(bayespval_yrange,na.rm=T),quantile(bayespval_yrange,c(0.01,0.1,0.25,0.5,0.75,0.9,0.99),na.rm=T),max(bayespval_yrange,na.rm=T),mean(bayespval_yrange,na.rm=T))
round(bayespval_yrange_summary,2)

# Predictive Concordance # 
# PC = matrix(NA,nrow=nyrs,ncol=nsites)
# for(i in 1:nsites)
# {
#   for(j in 1:nyrs)
#   {
#     quants = quantile(Y.posterior[j,i,],c(0.025,0.975))
#     PC[j,i] = Y[j,i]>quants[1] && Y[j,i]<quants[2]
#   }
# }
# 
# PC2 = apply(PC[1:nyrs,],2,function(x) sum(x,na.rm=T)/length(which(is.na(x)==FALSE)))
# pc2_summary = c(min(PC2,na.rm=T),quantile(PC2,c(0.01,0.1,0.25,0.5,0.75,0.9,0.99),na.rm=T),max(PC2,na.rm=T),mean(PC2,na.rm=T))
# round(pc2_summary,2)

print ("Stage 10 - Yrep simulation complete")

#------------------------------------------------------------#
# correlations - Obs #
N = 500
random_sims = sample(1:nsim,N,replace=FALSE)

cor_ndi = matrix(NA,nrow=nsites,ncol=1)
cor_gdd = matrix(NA,nrow=nsites,ncol=1)
cor_kdd = matrix(NA,nrow=nsites,ncol=1)

for (i in 1:nsites)
{
  dummy = length(which(Y[,i]>0))
  if (dummy >= 60)
  {
    cor_ndi[i,1] = cor(NDI[,i],Y[,i],use="pairwise.complete.obs",method="pearson")
    cor_gdd[i,1] = cor(GDD[,i],Y[,i],use="pairwise.complete.obs",method="pearson")
    cor_kdd[i,1] = cor(KDD[,i],Y[,i],use="pairwise.complete.obs",method="pearson")     
  }
}


# correlations - Post #
cor_ndi_post = matrix(NA,nrow=nsites,ncol=1)
cor_gdd_post = matrix(NA,nrow=nsites,ncol=1)
cor_kdd_post = matrix(NA,nrow=nsites,ncol=1)

for (i in 1:nsites)
{
  dummy = length(which(Y[,i]>0))
  if (dummy >= 60)
  {
    cor_ndi_post[i,1] = mean(cor(NDI[,i],Y.posterior[,i,random_sims],use="pairwise.complete.obs",method="pearson"))
    cor_gdd_post[i,1] = mean(cor(GDD[,i],Y.posterior[,i,random_sims],use="pairwise.complete.obs",method="pearson"))
    cor_kdd_post[i,1] = mean(cor(KDD[,i],Y.posterior[,i,random_sims],use="pairwise.complete.obs",method="pearson"))     
  }
}

cor_out = cbind(usa_fips, cor_ndi,cor_ndi_post, cor_gdd,cor_gdd_post, cor_kdd,cor_kdd_post)
write.table(cor_out,"cor_posteriorpredictivecheck.txt",row.names=T,col.names=T)

# Map of the correlation of predictors with log of the yield #

pdf("predictivecheck_map_correlations.pdf",width=10,height=8)
par(mfrow=c(3,2))

# Correlation with NDI - Obs #
plotvar = cor_ndi
nclr = 6
plotclr <- rev(brewer.pal(nclr,"RdBu"))
#class <- (classIntervals((plotvar), nclr, style="equal"))
class <- classIntervals(plotvar, nclr, style="fixed", fixedBreaks = c(-1, -0.5, -0.25, 0, 0.25, 0.5, 1))
colcode <- (findColours(class, plotclr))

map("usa")
map.axes(font=2)
plot(counties.shp,col=colcode, add=T,lty=0)
plot(state.shp,add=T,lwd=1,lty=6);
title(main="Correlation with NDI")
legend("bottomright", legend=names(attr(colcode, "table")), fill=attr(colcode, "palette"), cex=0.9, bty="n")

# Correlation with NDI - Post #
plotvar = cor_ndi_post
nclr = 6
plotclr <- rev(brewer.pal(nclr,"RdBu"))
#class <- (classIntervals((plotvar), nclr, style="equal"))
class <- classIntervals(plotvar, nclr, style="fixed", fixedBreaks = c(-1, -0.5, -0.25, 0, 0.25, 0.5, 1))
colcode <- (findColours(class, plotclr))

map("usa")
map.axes(font=2)
plot(counties.shp,col=colcode, add=T,lty=0)
plot(state.shp,add=T,lwd=1,lty=6);
title(main="Correlation with NDI")
legend("bottomright", legend=names(attr(colcode, "table")), fill=attr(colcode, "palette"), cex=0.9, bty="n")



# Correlation with GDD - Obs#
plotvar = cor_gdd
nclr = 6
plotclr <- rev(brewer.pal(nclr,"RdBu"))
#class <- (classIntervals((plotvar), nclr, style="equal"))
class <- classIntervals(plotvar, nclr, style="fixed", fixedBreaks = c(-1, -0.5, -0.25, 0, 0.25, 0.5, 1))
colcode <- (findColours(class, plotclr))

map("usa")
map.axes(font=2)
plot(counties.shp,col=colcode, add=T,lty=0)
plot(state.shp,add=T,lwd=1,lty=6);
title(main="Correlation with GDD")
legend("bottomright", legend=names(attr(colcode, "table")), fill=attr(colcode, "palette"), cex=0.9, bty="n")

# Correlation with GDD - Post #
plotvar = cor_gdd_post
nclr = 6
plotclr <- rev(brewer.pal(nclr,"RdBu"))
#class <- (classIntervals((plotvar), nclr, style="equal"))
class <- classIntervals(plotvar, nclr, style="fixed", fixedBreaks = c(-1, -0.5, -0.25, 0, 0.25, 0.5, 1))
colcode <- (findColours(class, plotclr))

map("usa")
map.axes(font=2)
plot(counties.shp,col=colcode, add=T,lty=0)
plot(state.shp,add=T,lwd=1,lty=6);
title(main="Correlation with GDD")
legend("bottomright", legend=names(attr(colcode, "table")), fill=attr(colcode, "palette"), cex=0.9, bty="n")


# Correlation with KDD - Obs #
plotvar = cor_kdd
nclr = 6
plotclr <- rev(brewer.pal(nclr,"RdBu"))
#class <- (classIntervals((plotvar), nclr, style="equal"))
class <- classIntervals(plotvar, nclr, style="fixed", fixedBreaks = c(-1, -0.5, -0.25, 0, 0.25, 0.5, 1))
colcode <- (findColours(class, plotclr))

map("usa")
map.axes(font=2)
plot(counties.shp,col=colcode, add=T,lty=0)
plot(state.shp,add=T,lwd=1,lty=6);
title(main="Correlation with KDD")
legend("bottomright", legend=names(attr(colcode, "table")), fill=attr(colcode, "palette"), cex=0.9, bty="n")

# Correlation with KDD - Post #
plotvar = cor_kdd_post
nclr = 6
plotclr <- rev(brewer.pal(nclr,"RdBu"))
#class <- (classIntervals((plotvar), nclr, style="equal"))
class <- classIntervals(plotvar, nclr, style="fixed", fixedBreaks = c(-1, -0.5, -0.25, 0, 0.25, 0.5, 1))
colcode <- (findColours(class, plotclr))

map("usa")
map.axes(font=2)
plot(counties.shp,col=colcode, add=T,lty=0)
plot(state.shp,add=T,lwd=1,lty=6);
title(main="Correlation with KDD")
legend("bottomright", legend=names(attr(colcode, "table")), fill=attr(colcode, "palette"), cex=0.9, bty="n")

### End of Plot ###
dev.off()

#------------------------------------------------------------#
